"use strict";
var path = require("path");
var fs = require("fs");
var main_1 = require("./../main");
function generateDefinitionFile() {
    var result = main_1.getInfoFromFiles([
        path.join(__dirname, "../../src/main.ts"),
        path.join(__dirname, "../../src/typings/index.d.ts")
    ], {
        showDebugMessages: false,
        compilerOptions: {
            strictNullChecks: true
        }
    });
    var fileInfo = result.getFile("main.ts");
    removeInternalProperties(result);
    fixSetType(result);
    var definitionFileText = fileInfo.writeExportsAsDefinitionFile({
        imports: [{
                defaultImport: "CodeBlockWriter",
                moduleSpecifier: "code-block-writer"
            }]
    });
    fs.writeFile(path.join(__dirname, "../../ts-type-info.d.ts"), definitionFileText);
}
exports.generateDefinitionFile = generateDefinitionFile;
function removeInternalProperties(result) {
    var baseDefClass = result.getFile("BaseDefinition.ts").getClass("BaseDefinition");
    baseDefClass.properties = baseDefClass.properties.filter(function (p) { return p.name !== "___uniqueID"; });
    var moduledDefinition = result.getFile("ModuledDefinition" + ".ts").getClass("ModuledDefinition");
    moduledDefinition.staticMethods = moduledDefinition.staticMethods.filter(function (p) { return p.name !== "initialize"; });
}
function fixSetType(result) {
    result.files.forEach(function (f) {
        f.classes.forEach(function (c) {
            if (c.properties.some(function (p) { return p.name === "setType"; })) {
                c.properties = c.properties.filter(function (p) { return p.name !== "setType"; });
                c.addMethod({
                    name: "setType",
                    returnType: "this",
                    overloadSignatures: [{
                            parameters: [
                                { name: "definition", type: "NamedDefinition" },
                                { name: "typeArguments", type: "string[]", isOptional: true }
                            ],
                            returnType: "this"
                        }, {
                            parameters: [{ name: "text", type: "string" }],
                            returnType: "this"
                        }]
                });
            }
        });
    });
}

//# sourceMappingURL=generateDefinitionFile.js.map
