"use strict";
var ts = require("typescript");
var path = require("path");
var errors_1 = require("./../errors");
var utils_1 = require("./../utils");
var TsTypeChecker_1 = require("./utils/TsTypeChecker");
var TsCache_1 = require("./utils/TsCache");
var TsSourceFile_1 = require("./TsSourceFile");
var TsMain = (function () {
    function TsMain(fileNames, options) {
        var _this = this;
        this.tsCache = new TsCache_1.TsCache();
        this.verifyFilesExist(fileNames);
        var compilerOptions = this.getTsCompilerOptions(options.compilerOptions);
        var program = ts.createProgram(fileNames, compilerOptions);
        var typeChecker = new TsTypeChecker_1.TsTypeChecker(program.getTypeChecker());
        this.tsSourceFiles = program.getSourceFiles().filter(function (file) {
            var baseName = path.basename(file.fileName);
            return baseName !== "lib.d.ts" && baseName !== "lib.es6.d.ts";
        }).map(function (sourceFile) {
            return new TsSourceFile_1.TsSourceFile({
                tsCache: _this.tsCache,
                sourceFile: sourceFile,
                symbol: typeChecker.getSymbolAtLocation(sourceFile),
                typeChecker: typeChecker
            });
        });
    }
    TsMain.prototype.getSourceFiles = function () {
        return this.tsSourceFiles;
    };
    TsMain.prototype.getTsCompilerOptions = function (compilerOptions) {
        function getValue(currentValue, newValue) {
            return (currentValue == null) ? newValue : currentValue;
        }
        var combinedOptions = (compilerOptions || {});
        combinedOptions.allowJs = getValue(combinedOptions.allowJs, true);
        combinedOptions.noLib = getValue(combinedOptions.noLib, false);
        combinedOptions.experimentalDecorators = getValue(combinedOptions.experimentalDecorators, true);
        combinedOptions.suppressExcessPropertyErrors = getValue(combinedOptions.suppressExcessPropertyErrors, true);
        combinedOptions.suppressImplicitAnyIndexErrors = getValue(combinedOptions.suppressImplicitAnyIndexErrors, true);
        combinedOptions.noImplicitAny = getValue(combinedOptions.noImplicitAny, false);
        combinedOptions.target = getValue(combinedOptions.target, ts.ScriptTarget.ES2015);
        combinedOptions.moduleResolution = getValue(combinedOptions.moduleResolution, ts.ModuleResolutionKind.NodeJs);
        combinedOptions.strictNullChecks = getValue(combinedOptions.strictNullChecks, false);
        return combinedOptions;
    };
    TsMain.prototype.verifyFilesExist = function (fileNames) {
        fileNames.forEach(function (fileName) {
            // unfortunately the ts compiler doesn't do things asynchronously so for now we won't either
            if (!utils_1.FileUtils.fileExistsSync(fileName)) {
                throw new errors_1.FileNotFoundError(utils_1.FileUtils.getAbsolutePath(fileName));
            }
        });
    };
    return TsMain;
}());
exports.TsMain = TsMain;

//# sourceMappingURL=TsMain.js.map
