"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var ts = require("typescript");
var ArrayUtils_1 = require("./../utils/ArrayUtils");
var decorators_1 = require("./../utils/decorators");
var Logger_1 = require("./../utils/Logger");
var TypeGuards_1 = require("./../utils/TypeGuards");
var definitions_1 = require("./../definitions");
var TsExpression_1 = require("./TsExpression");
var TsSignature_1 = require("./TsSignature");
var TsSymbol_1 = require("./TsSymbol");
var TsType_1 = require("./TsType");
var TsTypeNode_1 = require("./TsTypeNode");
var TsSourceFileChild_1 = require("./TsSourceFileChild");
var TsNode = (function (_super) {
    __extends(TsNode, _super);
    function TsNode(opts, originalSymbol) {
        _super.call(this, opts);
        this.originalSymbol = originalSymbol;
        if (opts.node == null) {
            throw new Error("Passed in " + "opts" + "." + "node" + " cannot be null.");
        }
        this.node = opts.node;
    }
    Object.defineProperty(TsNode.prototype, "tsSymbol", {
        get: function () {
            return this.originalSymbol || this.createSymbol(this.typeChecker.getSymbolAtLocation(this.node));
        },
        enumerable: true,
        configurable: true
    });
    TsNode.prototype.getName = function () {
        var name;
        var symbol = this.tsSymbol;
        if (symbol != null) {
            name = symbol.getName();
            if (name === "default") {
                var localSymbol = this.getLocalSymbol();
                if (localSymbol != null) {
                    name = localSymbol.getName();
                }
            }
        }
        return name || "";
    };
    TsNode.prototype.getSymbol = function () {
        return this.tsSymbol;
    };
    TsNode.prototype.getChildren = function () {
        var nodes = [];
        this.forEachChild(function (node) { return nodes.push(node); });
        return nodes;
    };
    TsNode.prototype.forEachChild = function (callback) {
        if (this.isNamespace()) {
            this.forEachLocalSymbol(callback);
        }
        else {
            this.forEachChildNode(callback);
        }
    };
    TsNode.prototype.getClassConstructorParameterScope = function () {
        var nodeFlags = this.node.flags;
        if ((nodeFlags & ts.NodeFlags.Private) !== 0) {
            return definitions_1.ClassConstructorParameterScope.Private;
        }
        else if ((nodeFlags & ts.NodeFlags.Protected) !== 0) {
            return definitions_1.ClassConstructorParameterScope.Protected;
        }
        else if ((nodeFlags & ts.NodeFlags.Public) !== 0) {
            return definitions_1.ClassConstructorParameterScope.Public;
        }
        else {
            return definitions_1.ClassConstructorParameterScope.None;
        }
    };
    TsNode.prototype.getConstantValue = function () {
        return this.typeChecker.getConstantValue(this.node);
    };
    TsNode.prototype.getDecoratorName = function () {
        var decorator = this.node;
        return this.getNameFromExpression(decorator.expression);
    };
    TsNode.prototype.getDecoratorArguments = function () {
        var decorator = this.node;
        return this.getArgumentsFromExpression(decorator.expression);
    };
    TsNode.prototype.getDecorators = function () {
        var _this = this;
        return this.node.decorators ? this.node.decorators.map(function (d) { return _this.createNode(d); }) : [];
    };
    TsNode.prototype.getDefaultExpression = function () {
        var propertyDeclaration = this.node;
        return propertyDeclaration.initializer != null ? this.createTsExpression(propertyDeclaration.initializer) : null;
    };
    TsNode.prototype.getDefaultImportNameAndSymbol = function () {
        var importDeclaration = this.node;
        var clause = importDeclaration.importClause;
        if (clause != null && clause.name != null) {
            return {
                name: clause.name.getText(),
                symbol: this.createSymbol(this.typeChecker.getAliasedSymbol(this.typeChecker.getSymbolAtLocation(clause.name)))
            };
        }
        else {
            return null;
        }
    };
    TsNode.prototype.getExpression = function () {
        var expression = null;
        var expressionStatement = this.node;
        if (expressionStatement.expression != null) {
            expression = this.createTsExpression(expressionStatement.expression);
        }
        return expression;
    };
    TsNode.prototype.getLocalSymbol = function () {
        return this.createSymbol(this.typeChecker.getLocalSymbolFromNode(this.node));
    };
    TsNode.prototype.getModuleSpecifierText = function () {
        var importDeclaration = this.node;
        if (importDeclaration.moduleSpecifier != null) {
            return (importDeclaration.moduleSpecifier.getText() || "").replace(/["']/g, "");
        }
        else {
            return null;
        }
    };
    TsNode.prototype.getFileNameOfModuleSpecifier = function () {
        var importDeclaration = this.node;
        var fileName = null;
        if (importDeclaration.moduleSpecifier != null) {
            var moduleSymbol = this.typeChecker.getSymbolAtLocation(importDeclaration.moduleSpecifier);
            if (moduleSymbol != null) {
                var sourceFile = this.typeChecker.getDeclarationFromSymbol(moduleSymbol).getSourceFile();
                fileName = sourceFile.fileName;
            }
        }
        return fileName;
    };
    TsNode.prototype.getModuleSpecifierSymbol = function () {
        var importDeclaration = this.node;
        return this.createSymbol(this.typeChecker.getSymbolAtLocation(importDeclaration.moduleSpecifier));
    };
    TsNode.prototype.getImportNamedImportNodes = function () {
        var _this = this;
        var importDeclaration = this.node;
        var importClause = (importDeclaration.importClause || {});
        var namedBindings = (importClause.namedBindings || {});
        return (namedBindings.elements || []).map(function (e) { return _this.createNode(e); });
    };
    TsNode.prototype.getReExportNamedExportNodes = function () {
        var _this = this;
        var exportDeclaration = this.node;
        var exportClause = exportDeclaration.exportClause || {};
        return (exportClause.elements || []).map(function (e) { return _this.createNode(e); });
    };
    TsNode.prototype.getNamespaceDeclarationType = function () {
        var nodeFlags = this.node.flags;
        if ((nodeFlags & ts.NodeFlags.Namespace) !== 0) {
            return definitions_1.NamespaceDeclarationType.Namespace;
        }
        else {
            return definitions_1.NamespaceDeclarationType.Module;
        }
    };
    TsNode.prototype.getParameterObjectBindingPatternElements = function () {
        var _this = this;
        var name = this.node.name;
        return (name.elements || []).map(function (e) { return _this.createNode(e); });
    };
    TsNode.prototype.getUserDefinedTypeGuardParameterName = function () {
        var typePredicateNode = this.node;
        return typePredicateNode.parameterName.text;
    };
    TsNode.prototype.getUserDefinedTypeGuardTypeNode = function () {
        var typeNode = this.node.type;
        return typeNode == null ? null : this.createTypeNode(typeNode);
    };
    TsNode.prototype.getParameters = function () {
        var _this = this;
        var parameters = this.node.parameters;
        return parameters.filter(function (p) { return p != null; }).map(function (p) { return _this.createNode(p); });
    };
    TsNode.prototype.getReturnType = function () {
        return this.getOrCreateType(this.typeChecker.getReturnTypeOfNode(this.node), this.node);
    };
    TsNode.prototype.getScope = function () {
        var nodeFlags = this.node.flags;
        if ((nodeFlags & ts.NodeFlags.Private) !== 0) {
            return definitions_1.Scope.Private;
        }
        else if ((nodeFlags & ts.NodeFlags.Protected) !== 0) {
            return definitions_1.Scope.Protected;
        }
        else {
            return definitions_1.Scope.Public;
        }
    };
    TsNode.prototype.getSignatureFromThis = function () {
        return new TsSignature_1.TsSignature({
            typeChecker: this.typeChecker,
            tsSourceFile: this.tsSourceFile,
            sourceFile: this.sourceFile,
            tsCache: this.tsCache,
            signature: this.typeChecker.getSignatureFromNode(this.node),
            node: this.node
        });
    };
    TsNode.prototype.getStarImportName = function () {
        var importDeclaration = this.node;
        if (importDeclaration.importClause != null) {
            var namespaceImport = importDeclaration.importClause.namedBindings;
            return namespaceImport.name.getText();
        }
        else {
            return null;
        }
    };
    TsNode.prototype.getStarSymbol = function () {
        var importDeclaration = this.node;
        if (importDeclaration.importClause != null) {
            var namespaceImport = importDeclaration.importClause.namedBindings;
            return this.createSymbol(this.typeChecker.getAliasedSymbol(this.typeChecker.getSymbolAtLocation(namespaceImport.name)));
        }
        else {
            return null;
        }
    };
    TsNode.prototype.getTypeParameters = function () {
        var _this = this;
        var typeParameteredDeclaration = this.node;
        return typeParameteredDeclaration.typeParameters ? typeParameteredDeclaration.typeParameters.map(function (typeParameter) { return _this.createNode(typeParameter); }) : [];
    };
    TsNode.prototype.getTypeParameterConstraintTypeNode = function () {
        var constraint = this.node.constraint;
        return constraint == null ? null : this.createTypeNode(constraint);
    };
    TsNode.prototype.getHeritageTypeNodes = function () {
        var _this = this;
        var clause = this.node;
        return clause.types ? clause.types.map(function (expressionWithTypeArgumentsTypeNode) { return _this.createTypeNode(expressionWithTypeArgumentsTypeNode); }) : [];
    };
    TsNode.prototype.getParent = function () {
        return this.node.parent == null ? null : this.createNode(this.node.parent);
    };
    TsNode.prototype.getType = function () {
        return this.getTypeAtLocationByNode(this.node);
    };
    TsNode.prototype.getTypeNode = function () {
        var typeNode = this.node["type"];
        return typeNode == null ? null : this.createTypeNode(typeNode);
    };
    TsNode.prototype.getText = function () {
        return this.node.getText();
    };
    TsNode.prototype.getVariableDeclarationType = function () {
        var nodeFlags = this.node.parent.flags;
        if (nodeFlags & ts.NodeFlags.Let) {
            return definitions_1.VariableDeclarationType.Let;
        }
        else if (nodeFlags & ts.NodeFlags.Const) {
            return definitions_1.VariableDeclarationType.Const;
        }
        else {
            return definitions_1.VariableDeclarationType.Var;
        }
    };
    TsNode.prototype.hasAbstractKeyword = function () {
        return this.hasModifierWithSyntaxKind(ts.SyntaxKind.AbstractKeyword);
    };
    TsNode.prototype.hasAsyncKeyword = function () {
        return this.hasModifierWithSyntaxKind(ts.SyntaxKind.AsyncKeyword);
    };
    TsNode.prototype.hasConstKeyword = function () {
        return this.hasModifierWithSyntaxKind(ts.SyntaxKind.ConstKeyword);
    };
    TsNode.prototype.hasDeclareKeyword = function () {
        return this.hasModifierWithSyntaxKind(ts.SyntaxKind.DeclareKeyword);
    };
    TsNode.prototype.hasExtendsKeyword = function () {
        return this.node.token === ts.SyntaxKind.ExtendsKeyword;
    };
    TsNode.prototype.hasFunctionBody = function () {
        return this.node.body != null;
    };
    TsNode.prototype.hasImplementsKeyword = function () {
        return this.node.token === ts.SyntaxKind.ImplementsKeyword;
    };
    TsNode.prototype.hasReadonlyKeyword = function () {
        return this.hasModifierWithSyntaxKind(ts.SyntaxKind.ReadonlyKeyword);
    };
    TsNode.prototype.hasStaticKeyword = function () {
        return this.hasModifierWithSyntaxKind(ts.SyntaxKind.StaticKeyword);
    };
    TsNode.prototype.isAmbient = function () {
        if (this.hasDeclareKeyword() || this.isInterface() || this.isTypeAlias()) {
            return true;
        }
        else {
            return this.isAnyParentAmbient();
        }
    };
    TsNode.prototype.isClass = function () {
        return this.getKind() === ts.SyntaxKind.ClassDeclaration;
    };
    TsNode.prototype.isConstructor = function () {
        return this.getKind() === ts.SyntaxKind.Constructor;
    };
    TsNode.prototype.isCallSignature = function () {
        return this.getKind() === ts.SyntaxKind.CallSignature;
    };
    TsNode.prototype.isConstructSignature = function () {
        return this.getKind() === ts.SyntaxKind.ConstructSignature;
    };
    TsNode.prototype.isDefaultKeyword = function () {
        return this.getKind() === ts.SyntaxKind.DefaultKeyword;
    };
    TsNode.prototype.isEnum = function () {
        return this.getKind() === ts.SyntaxKind.EnumDeclaration;
    };
    TsNode.prototype.isExportAssignment = function () {
        return this.getKind() === ts.SyntaxKind.ExportAssignment;
    };
    TsNode.prototype.isExportDeclaration = function () {
        return this.getKind() === ts.SyntaxKind.ExportDeclaration;
    };
    TsNode.prototype.isExportKeyword = function () {
        return this.getKind() === ts.SyntaxKind.ExportKeyword;
    };
    TsNode.prototype.isFunction = function () {
        return this.getKind() === ts.SyntaxKind.FunctionDeclaration;
    };
    TsNode.prototype.isFunctionType = function () {
        return this.getKind() === ts.SyntaxKind.FunctionType;
    };
    TsNode.prototype.isGeneratorFunction = function () {
        return this.node.asteriskToken != null;
    };
    TsNode.prototype.isGetAccessor = function () {
        return this.getKind() === ts.SyntaxKind.GetAccessor;
    };
    TsNode.prototype.isSetAccessor = function () {
        return this.getKind() === ts.SyntaxKind.SetAccessor;
    };
    TsNode.prototype.isHeritageClause = function () {
        return this.getKind() === ts.SyntaxKind.HeritageClause;
    };
    TsNode.prototype.isIdentifier = function () {
        return this.getKind() === ts.SyntaxKind.Identifier;
    };
    TsNode.prototype.isImport = function () {
        return this.getKind() === ts.SyntaxKind.ImportDeclaration;
    };
    TsNode.prototype.isIndexSignature = function () {
        return this.getKind() === ts.SyntaxKind.IndexSignature;
    };
    TsNode.prototype.isInterface = function () {
        return this.getKind() === ts.SyntaxKind.InterfaceDeclaration;
    };
    TsNode.prototype.isMethodDeclaration = function () {
        return this.getKind() === ts.SyntaxKind.MethodDeclaration;
    };
    TsNode.prototype.isMethodSignature = function () {
        return this.getKind() === ts.SyntaxKind.MethodSignature;
    };
    TsNode.prototype.isNamespace = function () {
        return this.getKind() === ts.SyntaxKind.ModuleDeclaration;
    };
    TsNode.prototype.isParameterOptional = function () {
        var parameterDeclaration = this.node;
        return parameterDeclaration.questionToken != null || parameterDeclaration.initializer != null || parameterDeclaration.dotDotDotToken != null;
    };
    TsNode.prototype.isPropertyOptional = function () {
        var propertyDeclaration = this.node;
        if (propertyDeclaration.questionToken != null) {
            return true;
        }
        else if (this.node.parent != null && this.node.parent.parent != null && this.node.parent.parent.kind === ts.SyntaxKind.Parameter) {
            // for parameter destructuring, check to see if the type has a questionToken
            var name_1 = this.getName();
            var prop = ArrayUtils_1.ArrayUtils.firstOrDefault(this.createNode(this.node.parent.parent).getType().getProperties(), function (p) { return p.getName() === name_1; });
            if (prop != null) {
                return prop.getOnlyNode().node.questionToken != null;
            }
        }
        return false;
    };
    TsNode.prototype.isPropertyDeclaration = function () {
        return this.getKind() === ts.SyntaxKind.PropertyDeclaration;
    };
    TsNode.prototype.isPropertySignature = function () {
        return this.getKind() === ts.SyntaxKind.PropertySignature;
    };
    TsNode.prototype.isRestParameter = function () {
        var parameterDeclaration = this.node;
        return parameterDeclaration.dotDotDotToken != null;
    };
    TsNode.prototype.isStarImport = function () {
        var importDeclaration = this.node;
        var namespaceImport = (importDeclaration.importClause == null) ? null : importDeclaration.importClause.namedBindings;
        return namespaceImport != null && namespaceImport.name != null;
    };
    TsNode.prototype.isTypeLiteral = function () {
        return this.getKind() === ts.SyntaxKind.TypeLiteral;
    };
    TsNode.prototype.isTypeAlias = function () {
        return this.getKind() === ts.SyntaxKind.TypeAliasDeclaration;
    };
    TsNode.prototype.isTypeParameter = function () {
        return this.getKind() === ts.SyntaxKind.TypeParameter;
    };
    TsNode.prototype.isVariable = function () {
        return this.getKind() === ts.SyntaxKind.VariableDeclaration;
    };
    TsNode.prototype.isUserDefinedTypeGuard = function () {
        return this.getKind() === ts.SyntaxKind.TypePredicate;
    };
    TsNode.prototype.nodeKindToString = function () {
        return this.typeChecker.getSyntaxKindAsString(this.getKind());
    };
    TsNode.prototype.getNamedImportName = function () {
        var name = this.node.name;
        return name != null ? name.text : null;
    };
    TsNode.prototype.getNamedImportPropertyName = function () {
        var propertyName = this.node.propertyName;
        return propertyName != null ? propertyName.text : null;
    };
    TsNode.prototype.getTypeAtLocationByNode = function (node) {
        return this.getOrCreateType(this.typeChecker.getTypeAtLocation(node), node);
    };
    TsNode.prototype.getOrCreateType = function (type, node) {
        var _this = this;
        return this.tsCache.getType(this.typeChecker, type, node, function () { return _this.createType(type, node); });
    };
    TsNode.prototype.createType = function (type, node) {
        return new TsType_1.TsType({
            sourceFile: this.sourceFile,
            typeChecker: this.typeChecker,
            type: type,
            node: node,
            tsCache: this.tsCache,
            tsSourceFile: this.tsSourceFile
        });
    };
    TsNode.prototype.createNode = function (node) {
        var _this = this;
        return this.tsCache.getNode(node, function () { return new TsNode({
            typeChecker: _this.typeChecker,
            sourceFile: _this.sourceFile,
            tsSourceFile: _this.tsSourceFile,
            tsCache: _this.tsCache,
            node: node
        }); });
    };
    TsNode.prototype.createTypeNode = function (node) {
        var _this = this;
        return this.tsCache.getTypeNode(node, function () { return new TsTypeNode_1.TsTypeNode({
            typeChecker: _this.typeChecker,
            sourceFile: _this.sourceFile,
            tsSourceFile: _this.tsSourceFile,
            tsCache: _this.tsCache,
            node: node
        }); });
    };
    TsNode.prototype.createSymbol = function (symbol) {
        var _this = this;
        if (symbol == null) {
            return null;
        }
        else {
            return this.tsCache.getSymbol(symbol, function () { return new TsSymbol_1.TsSymbol({
                typeChecker: _this.typeChecker,
                sourceFile: _this.sourceFile,
                tsSourceFile: _this.tsSourceFile,
                tsCache: _this.tsCache,
                symbol: symbol
            }); });
        }
    };
    TsNode.prototype.createTsExpression = function (expression) {
        return new TsExpression_1.TsExpression({
            typeChecker: this.typeChecker,
            sourceFile: this.sourceFile,
            tsCache: this.tsCache,
            expression: expression
        });
    };
    TsNode.prototype.forEachChildNode = function (callback) {
        var _this = this;
        ts.forEachChild(this.node, function (node) {
            if (_this.isNotDisallowedNode(node)) {
                if (node.kind === ts.SyntaxKind.VariableStatement) {
                    var declarationList = node.declarationList;
                    node = declarationList.declarations[0];
                    if (declarationList.declarations.length > 1) {
                        Logger_1.Logger.warn("Unknown situation where declaration list was greater than 1 for " + node.getText(_this.sourceFile));
                    }
                }
                callback(_this.createNode(node));
            }
        });
    };
    TsNode.prototype.forEachLocalSymbol = function (callback) {
        var _this = this;
        this.typeChecker.getLocalSymbolsFromNode(this.node).forEach(function (symbol) {
            (symbol.getDeclarations() || []).forEach(function (node) {
                callback(_this.createNode(node));
            });
        });
    };
    TsNode.prototype.isNotDisallowedNode = function (node) {
        return node.kind !== ts.SyntaxKind.EndOfFileToken;
    };
    TsNode.prototype.isAnyParentAmbient = function () {
        var declaration = this.node.parent;
        while (declaration != null) {
            if (declaration.flags & ts.NodeFlags.Ambient) {
                return true;
            }
            declaration = declaration.parent;
        }
        return false;
    };
    TsNode.prototype.getArgumentsFromExpression = function (expression) {
        var args = [];
        if (TypeGuards_1.TypeGuards.isCallExpression(expression)) {
            for (var _i = 0, _a = expression.arguments; _i < _a.length; _i++) {
                var arg = _a[_i];
                args.push(this.createTsExpression(arg));
            }
        }
        return args;
    };
    TsNode.prototype.getNameFromExpression = function (expression) {
        if (TypeGuards_1.TypeGuards.isLiteralExpression(expression)) {
            return expression.text;
        }
        else if (expression != null) {
            return this.getNameFromExpression(expression["expression"]);
        }
        else {
            Logger_1.Logger.warn("The expression was null");
            return "";
        }
    };
    TsNode.prototype.getKind = function () {
        return this.node.kind;
    };
    TsNode.prototype.hasModifierWithSyntaxKind = function (syntaxKind) {
        var node = (this.isVariable()) ? this.node.parent.parent : this.node;
        return node.modifiers ? node.modifiers.some(function (m) { return m.kind === syntaxKind; }) : false;
    };
    __decorate([
        decorators_1.Memoize
    ], TsNode.prototype, "tsSymbol", null);
    return TsNode;
}(TsSourceFileChild_1.TsSourceFileChild));
exports.TsNode = TsNode;

//# sourceMappingURL=TsNode.js.map
