"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ts = require("typescript");
var utils_1 = require("./../utils");
var TsNode_1 = require("./TsNode");
var TsSourceFileChild_1 = require("./TsSourceFileChild");
var TsSymbol = (function (_super) {
    __extends(TsSymbol, _super);
    function TsSymbol(opts) {
        _super.call(this, opts);
        if (opts.symbol == null) {
            throw new Error("Passed in ts.Symbol cannot be null.");
        }
        this.symbol = opts.symbol;
    }
    TsSymbol.prototype.getName = function () {
        return this.symbol.getName();
    };
    TsSymbol.prototype.getAliasSymbol = function () {
        var tsSymbol = this.typeChecker.getAliasedSymbol(this.symbol);
        return tsSymbol == null ? null : this.createSymbol(tsSymbol);
    };
    TsSymbol.prototype.getExportSymbols = function () {
        var _this = this;
        return Object.keys(this.symbol.exports).map(function (memberName) {
            var memberSymbol = _this.symbol.exports[memberName];
            return _this.createSymbol(memberSymbol);
        });
    };
    TsSymbol.prototype.getExportSymbolsByName = function () {
        var _this = this;
        var exportSymbols = {};
        Object.keys(this.symbol.exports).forEach(function (memberName) {
            var symbol = _this.createSymbol(_this.symbol.exports[memberName]);
            var isStarExport = memberName === "__export";
            if (isStarExport) {
                symbol.getNodes().forEach(function (node) {
                    var moduleSpecifierSymbol = node.getModuleSpecifierSymbol();
                    var starSymbolsByName = moduleSpecifierSymbol.getExportSymbolsByName();
                    Object.keys(starSymbolsByName).forEach(function (starSymbolMemberName) {
                        exportSymbols[starSymbolMemberName] = starSymbolsByName[starSymbolMemberName];
                    });
                });
            }
            else {
                exportSymbols[memberName] = _this.createSymbol(_this.symbol.exports[memberName]);
            }
        });
        return exportSymbols;
    };
    TsSymbol.prototype.getExportSymbolsOfModuleByName = function () {
        var _this = this;
        var symbolsByName = {};
        this.typeChecker.getExportsOfModule(this.symbol).map(function (symbol) { return _this.createSymbol(symbol); }).forEach(function (symbol) {
            var name = symbol.getName();
            if (symbol.isAlias()) {
                symbol = symbol.getAliasSymbol();
            }
            symbolsByName[name] = symbol;
        });
        return symbolsByName;
    };
    TsSymbol.prototype.getOnlyNode = function () {
        var nodes = this.getNodes();
        if (nodes.length === 0) {
            throw new Error("Symbol \"" + this.getName() + "\" was expected to have 1 node, but it had none.");
        }
        else if (nodes.length > 1) {
            utils_1.Logger.log("Symbol \"" + this.getName() + "\" was expected to have 1 node, but it had " + nodes.length + ".");
        }
        return nodes[0];
    };
    TsSymbol.prototype.getNodes = function () {
        var _this = this;
        return (this.symbol.getDeclarations() || []).map(function (declaration) { return _this.createNode(declaration); });
    };
    TsSymbol.prototype.isAlias = function () {
        return this.typeChecker.symbolHasFlag(this.symbol, ts.SymbolFlags.Alias);
    };
    TsSymbol.prototype.isDefaultExport = function () {
        return this.typeChecker.isSymbolDefaultExportOfFile(this.symbol, this.sourceFile);
    };
    TsSymbol.prototype.isExported = function () {
        var parentSymbol = this.getSymbolParent(this.symbol);
        if (parentSymbol == null) {
            return this.isDefaultExport();
        }
        else {
            return parentSymbol != null && parentSymbol.exports != null && parentSymbol.exports[this.symbol.name] != null;
        }
    };
    TsSymbol.prototype.isExportStar = function () {
        return (this.symbol.getFlags() & ts.SymbolFlags.ExportStar) !== 0;
    };
    TsSymbol.prototype.isNamedExport = function () {
        return this.typeChecker.isSymbolNamedExportOfFile(this.symbol, this.sourceFile);
    };
    TsSymbol.prototype.hasGetAccessor = function () {
        return (this.symbol.flags & ts.SymbolFlags.GetAccessor) !== 0;
    };
    TsSymbol.prototype.hasSetAccessor = function () {
        return (this.symbol.flags & ts.SymbolFlags.SetAccessor) !== 0;
    };
    TsSymbol.prototype.createSymbol = function (symbol) {
        var _this = this;
        return this.tsCache.getSymbol(symbol, function () { return new TsSymbol({
            sourceFile: _this.sourceFile,
            typeChecker: _this.typeChecker,
            tsCache: _this.tsCache,
            symbol: symbol,
            tsSourceFile: _this.tsSourceFile
        }); });
    };
    TsSymbol.prototype.createNode = function (node) {
        var _this = this;
        return this.tsCache.getNode(node, function () { return new TsNode_1.TsNode({
            sourceFile: _this.sourceFile,
            typeChecker: _this.typeChecker,
            tsCache: _this.tsCache,
            node: node,
            tsSourceFile: _this.tsSourceFile
        }, _this); });
    };
    TsSymbol.prototype.getSymbolParent = function (symbol) {
        return symbol == null ? null : symbol.parent;
    };
    return TsSymbol;
}(TsSourceFileChild_1.TsSourceFileChild));
exports.TsSymbol = TsSymbol;

//# sourceMappingURL=TsSymbol.js.map
