"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ts = require("typescript");
var utils_1 = require("./../utils");
var TsSourceFileChild_1 = require("./TsSourceFileChild");
var TsSymbol_1 = require("./TsSymbol");
var TsSignature_1 = require("./TsSignature");
var TsType = (function (_super) {
    __extends(TsType, _super);
    function TsType(opts) {
        _super.call(this, opts);
        this.type = opts.type;
        this.node = opts.node;
    }
    TsType.prototype.getText = function () {
        return this.typeChecker.typeToString(this.type, this.node);
    };
    TsType.prototype.getProperties = function () {
        var _this = this;
        var properties = this.type.getProperties();
        return (properties || []).filter(function (p) { return p.name !== "prototype"; }).map(function (property) { return _this.createSymbol(property); });
    };
    TsType.prototype.getCallSignatures = function () {
        var _this = this;
        var callSignatures = this.type.getCallSignatures();
        return (callSignatures || []).map(function (callSignature) { return _this.createTsSignature({ signature: callSignature }); });
    };
    TsType.prototype.getTypeArguments = function () {
        var _this = this;
        var tsTypeArguments = this.type.typeArguments || this.type.aliasTypeArguments || [];
        return tsTypeArguments.map(function (arg) { return utils_1.tryGet(_this.getText(), function () { return _this.createType(arg); }); }).filter(function (arg) { return arg != null; });
    };
    TsType.prototype.getSymbols = function () {
        var _this = this;
        var typeArray = this.type.types;
        if (this.type.aliasSymbol != null) {
            return [this.createSymbol(this.type.aliasSymbol)];
        }
        else if (!this.isEnumType() && typeArray != null) {
            return typeArray.map(function (t) { return t.symbol; }).filter(function (s) { return s != null; }).map(function (s) { return _this.createSymbol(s); });
        }
        else if (this.type.symbol != null) {
            return [this.createSymbol(this.type.symbol)];
        }
        else {
            return [];
        }
    };
    TsType.prototype.isArrayType = function () {
        return this.getArrayElementType() != null;
    };
    TsType.prototype.isEnumType = function () {
        return (this.type.flags & ts.TypeFlags.Enum) !== 0;
    };
    TsType.prototype.isTupleType = function () {
        return (this.type.flags & ts.TypeFlags.Tuple) !== 0;
    };
    TsType.prototype.isIntersectionType = function () {
        return (this.type.flags & ts.TypeFlags.Intersection) !== 0;
    };
    TsType.prototype.isUnionType = function () {
        return (this.type.flags & ts.TypeFlags.Union) !== 0;
    };
    TsType.prototype.isAnonymousType = function () {
        return (this.type.flags & ts.TypeFlags.Anonymous) !== 0;
    };
    TsType.prototype.isReferenceType = function () {
        return (this.type.flags & ts.TypeFlags.Reference) !== 0;
    };
    TsType.prototype.isClassType = function () {
        return (this.type.flags & ts.TypeFlags.Class) !== 0;
    };
    TsType.prototype.isInterfaceType = function () {
        return (this.type.flags & ts.TypeFlags.Interface) !== 0;
    };
    TsType.prototype.getUnionOrIntersectionTypes = function () {
        var _this = this;
        return (this.type.types || []).map(function (t) { return _this.createType(t); });
    };
    TsType.prototype.getArrayElementType = function () {
        var type = this.type.elementType || this.getArrayTypeArgument();
        return type == null ? null : this.createType(type);
    };
    TsType.prototype.getArrayTypeArgument = function () {
        var typeRef = this.type;
        if (typeRef.typeArguments != null && typeRef.typeArguments.length === 1 &&
            this.type.symbol != null && (this.type.symbol || {}).name === "Array") {
            return typeRef.typeArguments[0];
        }
        else {
            return null;
        }
    };
    TsType.prototype.createType = function (type) {
        var _this = this;
        return this.tsCache.getType(this.typeChecker, type, this.node, function () { return new TsType({
            sourceFile: _this.sourceFile,
            tsSourceFile: _this.tsSourceFile,
            typeChecker: _this.typeChecker,
            tsCache: _this.tsCache,
            type: type,
            node: _this.node
        }); });
    };
    TsType.prototype.createSymbol = function (symbol) {
        var _this = this;
        return this.tsCache.getSymbol(symbol, function () {
            return new TsSymbol_1.TsSymbol({
                sourceFile: _this.sourceFile,
                typeChecker: _this.typeChecker,
                tsCache: _this.tsCache,
                symbol: symbol,
                tsSourceFile: _this.tsSourceFile
            });
        });
    };
    TsType.prototype.createTsSignature = function (opts) {
        return new TsSignature_1.TsSignature({
            sourceFile: this.sourceFile,
            typeChecker: this.typeChecker,
            tsCache: this.tsCache,
            signature: opts.signature,
            node: this.node,
            tsSourceFile: this.tsSourceFile
        });
    };
    return TsType;
}(TsSourceFileChild_1.TsSourceFileChild));
exports.TsType = TsType;

//# sourceMappingURL=TsType.js.map
