"use strict";
var utils_1 = require("./../../utils");
var TsCache = (function () {
    function TsCache() {
        this.nodeCache = new utils_1.KeyValueCache();
        this.typeNodeCache = new utils_1.KeyValueCache();
        this.symbolCache = new utils_1.KeyValueCache();
        this.typeCacheContainer = new TypeCacheContainer();
    }
    TsCache.prototype.getSymbol = function (symbol, createFunc) {
        return this.symbolCache.getOrCreate(symbol, function () { return createFunc(); });
    };
    TsCache.prototype.getNode = function (node, createFunc) {
        return this.nodeCache.getOrCreate(node, function () { return createFunc(); });
    };
    TsCache.prototype.getTypeNode = function (node, createFunc) {
        return this.typeNodeCache.getOrCreate(node, function () { return createFunc(); });
    };
    TsCache.prototype.getType = function (typeChecker, type, node, createTsType) {
        var cache = this.typeCacheContainer.getCache(type);
        var typeText = typeChecker.typeToString(type, node);
        return cache.getOrCreate(typeText, function () { return createTsType(); });
    };
    return TsCache;
}());
exports.TsCache = TsCache;
var TypeCacheContainer = (function () {
    function TypeCacheContainer() {
        this.fileCache = new utils_1.KeyValueCache();
        this.typeCache = new utils_1.KeyValueCache();
    }
    TypeCacheContainer.prototype.getCache = function (tsType) {
        var fileName = this.getFileName(tsType);
        return fileName == null ? this.typeCache : this.getFileCache(fileName);
    };
    TypeCacheContainer.prototype.getFileCache = function (fileName) {
        return this.fileCache.getOrCreate(fileName, function () { return new utils_1.KeyValueCache(); });
    };
    TypeCacheContainer.prototype.getFileName = function (tsType) {
        var fileName = null;
        var symbol = tsType.getSymbol();
        if (symbol != null && symbol.valueDeclaration != null) {
            var sourceFile = symbol.valueDeclaration.getSourceFile();
            /* istanbul ignore else */
            if (sourceFile != null) {
                fileName = sourceFile.fileName;
            }
            else {
                utils_1.Logger.warn("Could not get source file.");
            }
        }
        return fileName;
    };
    return TypeCacheContainer;
}());

//# sourceMappingURL=TsCache.js.map
