"use strict";
var binders = require("./../binders");
var definitions = require("./../definitions");
var utils_1 = require("./../utils");
function bindToDefinition(binder, def) {
    binder.bind(def);
    return def;
}
var TsFactory = (function () {
    function TsFactory() {
        this.definitionByNode = new utils_1.KeyValueCache();
        this.files = new utils_1.KeyValueCache();
        this.deferredBindings = [];
        this.createdTypesWithDefinition = [];
    }
    TsFactory.prototype.getCallSignatureFromNode = function (node) {
        return this.getCallSignatureFromSignature(node.getSignatureFromThis());
    };
    TsFactory.prototype.getCallSignatureFromSignature = function (signature) {
        return bindToDefinition(new binders.TsCallSignatureBinder(this, signature), new definitions.CallSignatureDefinition());
    };
    TsFactory.prototype.getClassConstructor = function (node) {
        return bindToDefinition(new binders.TsClassConstructorBinder(this, node), new definitions.ClassConstructorDefinition());
    };
    TsFactory.prototype.getClassMethod = function (node) {
        return bindToDefinition(new binders.TsClassMethodBinder(this, node), new definitions.ClassMethodDefinition());
    };
    TsFactory.prototype.getClassStaticMethod = function (node) {
        return bindToDefinition(new binders.TsClassStaticMethodBinder(this, node), new definitions.ClassStaticMethodDefinition());
    };
    TsFactory.prototype.getClassProperty = function (nodes) {
        return bindToDefinition(new binders.TsClassPropertyBinder(this, nodes), new definitions.ClassPropertyDefinition());
    };
    TsFactory.prototype.getClassStaticProperty = function (node) {
        return bindToDefinition(new binders.TsClassStaticPropertyBinder(this, node), new definitions.ClassStaticPropertyDefinition());
    };
    TsFactory.prototype.getDecorator = function (node) {
        return bindToDefinition(new binders.TsDecoratorBinder(this, node), new definitions.DecoratorDefinition());
    };
    TsFactory.prototype.getEnumMember = function (node) {
        return bindToDefinition(new binders.TsEnumMemberBinder(node), new definitions.EnumMemberDefinition());
    };
    TsFactory.prototype.getExpression = function (tsExpression) {
        return bindToDefinition(new binders.TsExpressionBinder(tsExpression), new definitions.ExpressionDefinition());
    };
    TsFactory.prototype.getIndexSignatureFromNode = function (node) {
        return this.getIndexSignatureFromSignature(node.getSignatureFromThis());
    };
    TsFactory.prototype.getIndexSignatureFromSignature = function (signature) {
        return bindToDefinition(new binders.TsIndexSignatureBinder(this, signature), new definitions.IndexSignatureDefinition());
    };
    TsFactory.prototype.getInterfaceMethod = function (node) {
        return bindToDefinition(new binders.TsInterfaceMethodBinder(this, node), new definitions.InterfaceMethodDefinition());
    };
    TsFactory.prototype.getInterfaceProperty = function (node) {
        return bindToDefinition(new binders.TsInterfacePropertyBinder(this, node), new definitions.InterfacePropertyDefinition());
    };
    TsFactory.prototype.getObjectProperty = function (node) {
        return bindToDefinition(new binders.TsObjectPropertyBinder(this, node), new definitions.ObjectPropertyDefinition());
    };
    TsFactory.prototype.getNamedImportPart = function (node) {
        return bindToDefinition(new binders.TsNamedImportPartBinder(this, node), new definitions.NamedImportPartDefinition());
    };
    TsFactory.prototype.getDefaultImportPart = function (obj) {
        var def = new definitions.DefaultImportPartDefinition();
        def.name = obj.name;
        (_a = def.definitions).push.apply(_a, obj.definitions);
        def.expression = obj.expression;
        return def;
        var _a;
    };
    TsFactory.prototype.getStarImportPart = function (obj) {
        var def = new definitions.StarImportPartDefinition();
        def.name = obj.name;
        (_a = def.definitions).push.apply(_a, obj.definitions);
        def.expression = obj.expression;
        return def;
        var _a;
    };
    TsFactory.prototype.getTypeParameter = function (node) {
        return bindToDefinition(new binders.TsTypeParameterBinder(this, node), new definitions.TypeParameterDefinition());
    };
    TsFactory.prototype.getTypePropertyFromSymbol = function (symbol) {
        return this.getTypePropertyFromNode(symbol.getOnlyNode());
    };
    TsFactory.prototype.getTypePropertyFromNode = function (node) {
        return bindToDefinition(new binders.TsTypePropertyBinder(this, node), new definitions.TypePropertyDefinition());
    };
    TsFactory.prototype.getTypeNode = function (node) {
        var definition = bindToDefinition(new binders.TsTypeNodeBinder(this, node), new definitions.TypeNodeDefinition());
        this.createdTypesWithDefinition.push({
            type: node.getType(),
            definition: definition
        });
        return definition;
    };
    TsFactory.prototype.getTypeFromTypeNode = function (node) {
        var definition = bindToDefinition(new binders.TsTypeBinder(this, node.getType(), node), new definitions.TypeDefinition());
        this.createdTypesWithDefinition.push({
            type: node.getType(),
            definition: definition
        });
        return definition;
    };
    TsFactory.prototype.getType = function (type, node) {
        var definition = bindToDefinition(new binders.TsTypeBinder(this, type, node), new definitions.TypeDefinition());
        this.createdTypesWithDefinition.push({
            type: type,
            definition: definition
        });
        return definition;
    };
    TsFactory.prototype.getUserDefinedTypeGuardFromNode = function (node) {
        return bindToDefinition(new binders.TsUserDefinedTypeGuardBinder(this, node), new definitions.UserDefinedTypeGuardDefinition());
    };
    TsFactory.prototype.getAllExportableDefinitionsBySymbol = function (symbol) {
        symbol = symbol.isAlias() ? symbol.getAliasSymbol() : symbol;
        var defs = this.getAllDefinitionsBySymbol(symbol);
        var exportableDefinitions = [];
        function handleDefinition(definition) {
            if (definition instanceof definitions.ReExportDefinition) {
                handleReExport(definition);
            }
            else {
                exportableDefinitions.push(definition);
            }
        }
        function handleReExport(reExportDefinition) {
            reExportDefinition.getExports().forEach(handleDefinition);
        }
        defs.forEach(handleDefinition);
        return exportableDefinitions;
    };
    TsFactory.prototype.getAllDefinitionsBySymbol = function (symbol) {
        var _this = this;
        return symbol.getNodes().map(function (node) {
            if (node.isTypeLiteral()) {
                var parentNode = node.getParent();
                if (parentNode != null && parentNode.isTypeAlias())
                    node = parentNode;
            }
            return _this.getDefinitionByNode(node);
        }).filter(function (d) { return d != null; });
    };
    TsFactory.prototype.getDefinitionByNode = function (node) {
        return this.definitionByNode.get(node) || this.createDefinition(node);
    };
    TsFactory.prototype.getFileDefinition = function (file) {
        var _this = this;
        return this.files.getOrCreate(file, function () {
            var def = new definitions.FileDefinition();
            var binder = new binders.TsFileBinder(_this, file);
            binder.bind(def);
            return def;
        });
    };
    TsFactory.prototype.getDefinitionsOrExpressionFromExportSymbol = function (symbol) {
        var obj = { definitions: [], expression: null };
        if (symbol != null) {
            if (symbol.isAlias()) {
                symbol = symbol.getAliasSymbol();
            }
            var expression = this.getExpressionFromExportSymbol(symbol);
            if (expression != null) {
                obj.expression = expression;
            }
            else {
                (_a = obj.definitions).push.apply(_a, (this.getAllDefinitionsBySymbol(symbol)));
            }
        }
        return obj;
        var _a;
    };
    TsFactory.prototype.getExpressionFromExportSymbol = function (symbol) {
        var nodes = symbol.getNodes();
        if (nodes.length === 1) {
            var tsExpression = nodes[0].getExpression();
            if (tsExpression != null) {
                return this.getExpression(tsExpression);
            }
        }
        return null;
    };
    TsFactory.prototype.fillAllCachedTypesWithDefinitions = function () {
        var _this = this;
        this.createdTypesWithDefinition.forEach(function (typeAndDef) {
            var type = typeAndDef.type, definition = typeAndDef.definition;
            var symbols = type.getSymbols();
            symbols.forEach(function (s) {
                var hasChildTypes = definition.unionTypes.length > 0 || definition.intersectionTypes.length > 0 || definition.isArrayType();
                if (!hasChildTypes) {
                    (_a = definition.definitions).push.apply(_a, (_this.getAllDefinitionsBySymbol(s)));
                }
                var _a;
            });
        });
    };
    TsFactory.prototype.bindDeferred = function () {
        this.deferredBindings.forEach(function (obj) {
            obj.binder.bind(obj.definition);
        });
    };
    TsFactory.prototype.createDefinition = function (node) {
        var definition = null;
        // todo: all these if statements are very similar. Need to reduce the redundancy
        if (node.isFunction()) {
            definition = bindToDefinition(new binders.TsFunctionBinder(this, node), new definitions.FunctionDefinition());
        }
        else if (node.isClass()) {
            definition = bindToDefinition(new binders.TsClassBinder(this, node), new definitions.ClassDefinition());
        }
        else if (node.isInterface()) {
            definition = bindToDefinition(new binders.TsInterfaceBinder(this, node), new definitions.InterfaceDefinition());
        }
        else if (node.isEnum()) {
            definition = bindToDefinition(new binders.TsEnumBinder(this, node), new definitions.EnumDefinition());
        }
        else if (node.isVariable()) {
            definition = bindToDefinition(new binders.TsVariableBinder(this, node), new definitions.VariableDefinition());
        }
        else if (node.isTypeAlias()) {
            definition = bindToDefinition(new binders.TsTypeAliasBinder(this, node), new definitions.TypeAliasDefinition());
        }
        else if (node.isNamespace()) {
            definition = bindToDefinition(new binders.TsNamespaceBinder(this, node), new definitions.NamespaceDefinition());
        }
        else if (node.isExportDeclaration()) {
            var binder = new binders.TsReExportBinder(this, node);
            definition = new definitions.ReExportDefinition();
            this.deferredBindings.push({
                binder: binder,
                definition: definition
            });
        }
        else if (node.isImport()) {
            var binder = new binders.TsImportBinder(this, node);
            definition = new definitions.ImportDefinition();
            this.deferredBindings.push({
                binder: binder,
                definition: definition
            });
        }
        else if (node.isExportAssignment()) {
        }
        else if (node.isTypeParameter()) {
        }
        else if (node.isMethodSignature() || node.isFunctionType()) {
        }
        else {
            utils_1.Logger.warn("Unknown node kind: " + node.nodeKindToString());
        }
        if (definition != null) {
            this.definitionByNode.add(node, definition);
        }
        return definition;
    };
    return TsFactory;
}());
exports.TsFactory = TsFactory;

//# sourceMappingURL=TsFactory.js.map
