"use strict";
var assert = require("assert");
var main_1 = require("./../../../main");
describe("ExportableDefinition", function () {
    describe("named exports", function () {
        var code = "\nexport namespace MyNamespace {\n}\nexport module MyModule {\n}\nexport interface MyInterface {\n}\nexport class MyClass {\n}\nexport enum MyEnum {\n}\nexport function myFunction() {\n}\n";
        var myFile = main_1.getInfoFromString(code);
        describe("write()", function () {
            it("should contain the file written out with export keywords", function () {
                var expected = "export namespace MyNamespace {\n}\n\nexport module MyModule {\n}\n\nexport interface MyInterface {\n}\n\nexport class MyClass {\n}\n\nexport enum MyEnum {\n}\n\nexport function myFunction() {\n}\n";
                assert.equal(myFile.write(), expected);
            });
        });
    });
    describe("default export statements", function () {
        it("should write export default for a namespace", function () {
            var code = "namespace MyNamespace {}\nexport default MyNamespace;";
            var expected = "namespace MyNamespace {\n}\n\nexport default MyNamespace;\n";
            assert.equal(main_1.getInfoFromString(code).write(), expected);
        });
        it("should write export default for a module", function () {
            var code = "module MyModule {}\nexport default MyModule;";
            var expected = "module MyModule {\n}\n\nexport default MyModule;\n";
            assert.equal(main_1.getInfoFromString(code).write(), expected);
        });
        it("should write export default for an interface", function () {
            var code = "interface MyInterface {}\nexport default MyInterface;";
            var expected = "interface MyInterface {\n}\n\nexport default MyInterface;\n";
            assert.equal(main_1.getInfoFromString(code).write(), expected);
        });
        it("should write export default for a class", function () {
            var code = "class MyClass {}\nexport default MyClass;";
            var expected = "class MyClass {\n}\n\nexport default MyClass;\n";
            assert.equal(main_1.getInfoFromString(code).write(), expected);
        });
        it("should write export default for an enum", function () {
            var code = "enum MyEnum {}\nexport default MyEnum;";
            var expected = "enum MyEnum {\n}\n\nexport default MyEnum;\n";
            assert.equal(main_1.getInfoFromString(code).write(), expected);
        });
        it("should write export default for a function", function () {
            var code = "function MyFunction() {}\nexport default MyFunction;";
            var expected = "function MyFunction() {\n}\n\nexport default MyFunction;\n";
            assert.equal(main_1.getInfoFromString(code).write(), expected);
        });
    });
});

//# sourceMappingURL=exportableDefinitionWriteTests.js.map
