"use strict";
var assert = require("assert");
var main_1 = require("./../../../main");
var code = "\nfunction myFunction(str: string) {\n    return \"\";\n}\nfunction myFunction2<T extends string, U>(str: T, num: U) {\n}\nexport function myMultipleSignatures<T>(): string;\nexport function myMultipleSignatures<T>(str?: string) {\n    return \"\";\n}\ndeclare function myDeclareMultipleSignatures(): string;\ndeclare function myDeclareMultipleSignatures(str?: string): string;\nfunction myFunctionBodyWriter() {\n    return \"\";\n}\ndeclare async function myDeclareAsyncFunction();\nexport async function myExportAsyncFunction() {\n}\nfunction myFunctionWithDestructuringParameter(str: string, { prop1, prop2 = 4 }: { prop1: number; prop2?: number; }) {\n}\nfunction myFunctionWithThisType(this: string, num: number) {\n}\nfunction *myGeneratorFunction() {\n    yield return \"string\";\n}\nfunction myParameterFunction(param1: string, param2 = \"text\", param3?: number, ...params: string[]) {\n}\nfunction myTypeGuard(p: string): p is string {\n}\n";
describe("FunctionDefinition", function () {
    var file = main_1.getInfoFromString(code);
    describe("write()", function () {
        describe("myFunction", function () {
            it("should contain the function written out", function () {
                var expected = "function myFunction(str: string) {\n}\n";
                assert.equal(file.functions[0].write(), expected);
            });
        });
        describe("myFunction2", function () {
            it("should contain the function written out", function () {
                var expected = "function myFunction2<T extends string, U>(str: T, num: U) {\n}\n";
                assert.equal(file.functions[1].write(), expected);
            });
        });
        describe("myMultipleSignatures", function () {
            it("should contain the function written out", function () {
                var expected = "export function myMultipleSignatures<T>(): string;\nexport function myMultipleSignatures<T>(str?: string): string {\n}\n";
                assert.equal(file.functions[2].write(), expected);
            });
        });
        describe("myDeclareMultipleSignatures", function () {
            it("should contain the function written out", function () {
                var expected = "declare function myDeclareMultipleSignatures(): string;\ndeclare function myDeclareMultipleSignatures(str?: string): string;\n";
                assert.equal(file.functions[3].write(), expected);
            });
        });
        describe("onWriteFunctionBody", function () {
            var funcDef = file.functions[4];
            funcDef.onWriteFunctionBody = function (writer) {
                writer.write("return \"text\";");
            };
            it("should write the function with a body", function () {
                var expected = "function myFunctionBodyWriter() {\n    return \"text\";\n}\n";
                assert.equal(funcDef.write(), expected);
            });
        });
        describe("myDeclareAsyncFunction", function () {
            it("should contain the function written out", function () {
                var expected = "declare async function myDeclareAsyncFunction(): any;\n";
                assert.equal(file.functions[5].write(), expected);
            });
        });
        describe("myExportAsyncFunction", function () {
            it("should contain the function written out", function () {
                var expected = "export async function myExportAsyncFunction() {\n}\n";
                assert.equal(file.functions[6].write(), expected);
            });
        });
        describe("myFunctionWithDestructuringParameter", function () {
            it("should contain the function written out", function () {
                var expected = "function myFunctionWithDestructuringParameter(str: string, { prop1, prop2 = 4 }: { prop1: number; prop2?: number; }) {\n}\n";
                assert.equal(file.functions[7].write(), expected);
            });
        });
        describe("myFunctionWithThisType", function () {
            it("should contain the function written out", function () {
                var expected = "function myFunctionWithThisType(this: string, num: number) {\n}\n";
                assert.equal(file.functions[8].write(), expected);
            });
        });
        describe("myGeneratorFunction", function () {
            it("should contain the function written out", function () {
                var expected = "function *myGeneratorFunction() {\n}\n";
                assert.equal(file.functions[9].write(), expected);
            });
        });
        describe("myParameterFunction", function () {
            it("should contain the function written out", function () {
                var expected = "function myParameterFunction(param1: string, param2 = \"text\", param3?: number, ...params: string[]) {\n}\n";
                assert.equal(file.functions[10].write(), expected);
            });
        });
        describe("myTypeGuard", function () {
            it("should contain the function written out", function () {
                var expected = "function myTypeGuard(p: string): p is string {\n}\n";
                assert.equal(file.functions[11].write(), expected);
            });
        });
    });
});

//# sourceMappingURL=functionWriteTests.js.map
