"use strict";
var main_1 = require("./../../../main");
var definitions_1 = require("./../../../definitions");
var testHelpers_1 = require("./../../testHelpers");
describe("class method", function () {
    var code = "\nclass MyClass {\n    // parameters\n    myParameterMethod(myParameter: string, myDefaultParameter = 15, myOptionalParameter?: string, ...myRestParameter: number[]) {\n        return \"\";\n    }\n\n    myThisTypeMethod(this: string, num: number) {\n    }\n\n    // return types\n    myExplicitReturnTypeMethod(): string {\n        return null;\n    }\n\n    myImplicitReturnTypeMethod() {\n        return \"\";\n    }\n\n    // scope\n    public myPublicMethod() {\n    }\n\n    protected myProtectedMethod() {\n    }\n\n    private myPrivateMethod() {\n    }\n\n    // async\n    async myAsyncMethod(): any {\n    }\n\n    // user defined type guard\n    myUserDefinedTypeGuard(): this is MyClass {\n    }\n}";
    var def = main_1.getInfoFromString(code);
    testHelpers_1.runFileDefinitionTests(def, {
        classes: [{
                name: "MyClass",
                methods: [{
                        name: "myParameterMethod",
                        parameters: [{
                                name: "myParameter",
                                type: { text: "string" }
                            }, {
                                name: "myDefaultParameter",
                                type: { text: "number" },
                                isOptional: true,
                                defaultExpression: { text: "15" }
                            }, {
                                name: "myOptionalParameter",
                                type: { text: "string" },
                                isOptional: true
                            }, {
                                name: "myRestParameter",
                                type: { text: "number[]", isArrayType: true, arrayElementType: { text: "number" } },
                                isOptional: true,
                                isRestParameter: true
                            }],
                        returnType: { text: "string" }
                    }, {
                        name: "myThisTypeMethod",
                        thisType: { text: "string" },
                        parameters: [{
                                name: "num",
                                type: { text: "number" }
                            }]
                    }, {
                        name: "myExplicitReturnTypeMethod",
                        returnType: { text: "string" }
                    }, {
                        name: "myImplicitReturnTypeMethod",
                        returnType: { text: "string" }
                    }, {
                        name: "myPublicMethod",
                        scope: definitions_1.Scope.Public
                    }, {
                        name: "myProtectedMethod",
                        scope: definitions_1.Scope.Protected
                    }, {
                        name: "myPrivateMethod",
                        scope: definitions_1.Scope.Private
                    }, {
                        name: "myAsyncMethod",
                        isAsync: true,
                        returnType: { text: "any" }
                    }, {
                        name: "myUserDefinedTypeGuard",
                        userDefinedTypeGuard: {
                            type: "MyClass"
                        },
                        returnType: {
                            text: "this is MyClass"
                        }
                    }]
            }]
    });
});

//# sourceMappingURL=methodTests.js.map
