"use strict";
var main_1 = require("./../../../main");
var definitions_1 = require("./../../../definitions");
var testHelpers_1 = require("./../../testHelpers");
describe("class property decorator tests", function () {
    var code = "\nfunction MyClassPropertyDecorator(target: Object, propertyKey: string) {\n}\n\nfunction MyClassPropertyAccessorDecorator(target: Object, propertyKey: string, descriptor: TypedPropertyDescriptor<any>) {\n    return descriptor;\n}\n\nclass MyClass {\n    @MyClassPropertyDecorator\n    myProperty1: string;\n\n    @MyClassPropertyAccessorDecorator\n    get myProperty2() {\n        return \"\";\n    }\n\n    @MyClassPropertyAccessorDecorator\n    set myProperty3(val: string) {\n    }\n\n    get myProperty3() {\n        return \"\";\n    }\n}\n";
    var def = main_1.getInfoFromString(code);
    testHelpers_1.runFileDefinitionTests(def, {
        functions: [{
                name: "MyClassPropertyDecorator",
                parameters: [{
                        name: "target",
                        type: {
                            text: "Object"
                        }
                    }, {
                        name: "propertyKey",
                        type: {
                            text: "string"
                        }
                    }]
            }, {
                name: "MyClassPropertyAccessorDecorator",
                parameters: [{
                        name: "target",
                        type: {
                            text: "Object"
                        }
                    }, {
                        name: "propertyKey",
                        type: {
                            text: "string"
                        }
                    }, {
                        name: "descriptor",
                        type: {
                            text: "TypedPropertyDescriptor<any>"
                        }
                    }],
                returnType: {
                    text: "TypedPropertyDescriptor<any>"
                }
            }],
        classes: [{
                name: "MyClass",
                properties: [{
                        name: "myProperty1",
                        type: {
                            text: "string"
                        },
                        decorators: [{
                                name: "MyClassPropertyDecorator"
                            }]
                    }, {
                        name: "myProperty2",
                        type: {
                            text: "string"
                        },
                        decorators: [{
                                name: "MyClassPropertyAccessorDecorator"
                            }],
                        kind: definitions_1.ClassPropertyKind.GetAccessor
                    }, {
                        name: "myProperty3",
                        type: {
                            text: "string"
                        },
                        decorators: [{
                                name: "MyClassPropertyAccessorDecorator"
                            }],
                        kind: definitions_1.ClassPropertyKind.GetSetAccessor
                    }]
            }]
    });
});

//# sourceMappingURL=propertyDecoratorTests.js.map
