"use strict";
var main_1 = require("./../../../main");
var definitions_1 = require("./../../../definitions");
var testHelpers_1 = require("./../../testHelpers");
describe("class property tests", function () {
    var code = "\nclass MyClass {\n    readonly myString: string;\n    myImplicit = 4;\n    myAny;\n    myOptional?: string;\n\n    public myExplicitPublic;\n    protected myProtected;\n    private myPrivate;\n\n    get myGetAccessor() {\n        return \"\";\n    }\n\n    set mySetAccessor(val: string) {\n    }\n\n    get myGetAndSetAccessor() {\n        return \"\";\n    }\n    set myGetAndSetAccessor(val: string) {\n    }\n\n    abstract myAbstractProperty: string;\n}";
    var def = main_1.getInfoFromString(code);
    testHelpers_1.runFileDefinitionTests(def, {
        classes: [{
                name: "MyClass",
                properties: [{
                        name: "myString",
                        isReadonly: true,
                        type: { text: "string" }
                    }, {
                        name: "myImplicit",
                        type: { text: "number" },
                        defaultExpression: { text: "4" }
                    }, {
                        name: "myAny",
                        type: { text: "any" }
                    }, {
                        name: "myOptional",
                        type: { text: "string" },
                        isOptional: true
                    }, {
                        name: "myExplicitPublic",
                        type: { text: "any" },
                        scope: definitions_1.Scope.Public
                    }, {
                        name: "myProtected",
                        type: { text: "any" },
                        scope: definitions_1.Scope.Protected
                    }, {
                        name: "myPrivate",
                        type: { text: "any" },
                        scope: definitions_1.Scope.Private
                    }, {
                        name: "myGetAccessor",
                        type: { text: "string" },
                        kind: definitions_1.ClassPropertyKind.GetAccessor
                    }, {
                        name: "mySetAccessor",
                        type: { text: "string" },
                        kind: definitions_1.ClassPropertyKind.SetAccessor
                    }, {
                        name: "myGetAndSetAccessor",
                        type: { text: "string" },
                        kind: definitions_1.ClassPropertyKind.GetSetAccessor
                    }, {
                        name: "myAbstractProperty",
                        isAbstract: true,
                        type: { text: "string" }
                    }]
            }]
    });
});

//# sourceMappingURL=propertyTests.js.map
