"use strict";
var main_1 = require("./../../../main");
var definitions_1 = require("./../../../definitions");
var testHelpers_1 = require("./../../testHelpers");
describe("class static method", function () {
    var code = "\nclass MyClassWithStaticMethods {\n    // parameters\n    static myParameterMethod(myParameter: string, myDefaultParameter = \"some string\", myOptionalParameter?: string, ...myRestParameter: number[]) {\n        return \"\";\n    }\n\n    static myThisTypeMethod(this: string, num: number) {\n    }\n\n    // return types\n    static myExplicitReturnTypeMethod(): string {\n        return null;\n    }\n\n    static myImplicitReturnTypeMethod() {\n        return \"\";\n    }\n\n    // scope\n    public static myPublicMethod() {\n    }\n\n    protected static myProtectedMethod() {\n    }\n\n    private static myPrivateMethod() {\n    }\n\n    // async\n    static async myAsyncMethod(): any {\n    }\n}";
    var def = main_1.getInfoFromString(code);
    testHelpers_1.runFileDefinitionTests(def, {
        classes: [{
                name: "MyClassWithStaticMethods",
                staticMethods: [{
                        name: "myParameterMethod",
                        parameters: [{
                                name: "myParameter",
                                type: { text: "string" }
                            }, {
                                name: "myDefaultParameter",
                                type: { text: "string" },
                                isOptional: true,
                                defaultExpression: { text: "\"some string\"" }
                            }, {
                                name: "myOptionalParameter",
                                type: { text: "string" },
                                isOptional: true
                            }, {
                                name: "myRestParameter",
                                type: {
                                    text: "number[]",
                                    isArrayType: true,
                                    arrayElementType: { text: "number" }
                                },
                                isOptional: true,
                                isRestParameter: true
                            }],
                        returnType: { text: "string" }
                    }, {
                        name: "myThisTypeMethod",
                        thisType: { text: "string" },
                        parameters: [{
                                name: "num",
                                type: { text: "number" }
                            }]
                    }, {
                        name: "myExplicitReturnTypeMethod",
                        returnType: { text: "string" }
                    }, {
                        name: "myImplicitReturnTypeMethod",
                        returnType: { text: "string" }
                    }, {
                        name: "myPublicMethod",
                        scope: definitions_1.Scope.Public
                    }, {
                        name: "myProtectedMethod",
                        scope: definitions_1.Scope.Protected
                    }, {
                        name: "myPrivateMethod",
                        scope: definitions_1.Scope.Private
                    }, {
                        name: "myAsyncMethod",
                        isAsync: true,
                        returnType: { text: "any" }
                    }]
            }]
    });
});

//# sourceMappingURL=staticMethodTests.js.map
