"use strict";
var path = require("path");
var assert = require("assert");
var main_1 = require("./../../../main");
var testHelpers_1 = require("./../../testHelpers");
var definitions_1 = require("./../../../definitions");
describe("file import tests", function () {
    var fileName = path.join(__dirname, "../../../../src/tests/languageTests/file/testFiles/import.ts");
    var fileDef = main_1.getInfoFromFiles([fileName]).getFile("import.ts");
    var i = 0;
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./defaultExport",
        fileName: "defaultExport.ts",
        defaultImport: {
            name: "DefaultExport",
            definitions: [{
                    name: "DefaultExport",
                    type: definitions_1.ClassDefinition
                }]
        }
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./defaultExportSeparate",
        fileName: "defaultExportSeparate.ts",
        defaultImport: {
            name: "DefaultExportSeparate",
            definitions: [{
                    name: "DefaultExportSeparate",
                    type: definitions_1.ClassDefinition
                }]
        }
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./namedExports",
        fileName: "namedExports.ts",
        namedImports: [{
                name: "NamedExport1",
                definitions: [{
                        name: "NamedExport1",
                        type: definitions_1.ClassDefinition
                    }, {
                        name: "NamedExport1",
                        type: definitions_1.NamespaceDefinition
                    }]
            }, {
                name: "NamedExport2",
                definitions: [{
                        name: "NamedExport2",
                        type: definitions_1.ClassDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./namedExports",
        fileName: "namedExports.ts",
        starImportName: "NamedExports",
        starImports: [{
                name: "NamedExport1",
                definitions: [{
                        name: "NamedExport1",
                        type: definitions_1.ClassDefinition
                    }, {
                        name: "NamedExport1",
                        type: definitions_1.NamespaceDefinition
                    }]
            }, {
                name: "NamedExport2",
                definitions: [{
                        name: "NamedExport2",
                        type: definitions_1.ClassDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./namedWithDefaultExport",
        fileName: "namedWithDefaultExport.ts",
        defaultImport: {
            name: "DefaultExportClassLocalName1",
            definitions: [{
                    name: "DefaultExportClass",
                    type: definitions_1.ClassDefinition
                }]
        },
        namedImports: [{
                name: "NamedExportEnum",
                alias: "NamedExportLocalName",
                definitions: [{
                        name: "NamedExportEnum",
                        type: definitions_1.EnumDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./namedWithDefaultExport",
        fileName: "namedWithDefaultExport.ts",
        defaultImport: {
            name: "DefaultExportClassLocalName2",
            definitions: [{
                    name: "DefaultExportClass",
                    type: definitions_1.ClassDefinition
                }]
        },
        starImportName: "NamedWithDefaultExportStarImport",
        starImports: [{
                name: "NamedExportEnum",
                definitions: [{
                        name: "NamedExportEnum",
                        type: definitions_1.EnumDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./expression",
        fileName: "expression.ts",
        defaultImport: {
            name: "Expression",
            expression: { text: "\"test string\"" }
        }
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./expression",
        fileName: "expression.ts"
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./reExport",
        fileName: "reExport.ts",
        starImportName: "reexports",
        starImports: [{
                name: "RenamedExport",
                definitions: [{
                        name: "NamedExport1",
                        type: definitions_1.ClassDefinition
                    }, {
                        name: "NamedExport1",
                        type: definitions_1.NamespaceDefinition
                    }]
            }, {
                name: "NamedExport1",
                definitions: [{
                        name: "NamedExport1",
                        type: definitions_1.ClassDefinition
                    }, {
                        name: "NamedExport1",
                        type: definitions_1.NamespaceDefinition
                    }]
            }, {
                name: "NamedExport2",
                definitions: [{
                        name: "NamedExport2",
                        type: definitions_1.ClassDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "definition",
        fileName: "definition.d.ts",
        starImportName: "definitionLocal",
        starImports: [{
                name: "DefinitionInterface",
                definitions: [{
                        name: "DefinitionInterface",
                        type: definitions_1.InterfaceDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./definitionNamespace",
        fileName: "definitionNamespace.d.ts",
        starImportName: "definitionNamespace",
        starImports: [{
                name: "DefinitionNamespaceInterface",
                definitions: [{
                        name: "DefinitionNamespaceInterface",
                        type: definitions_1.InterfaceDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "definition-var",
        fileName: "definition-var.d.ts",
        starImportName: "definitionVar",
        starImports: [{
                name: "Methods",
                definitions: [{
                        name: "Methods",
                        type: definitions_1.VariableDefinition
                    }]
            }]
    });
    testHelpers_1.runImportDefinitionTests(fileDef.imports[i++], {
        moduleSpecifier: "./namedExpression",
        fileName: "namedExpression.ts",
        starImportName: "NamedExpressions",
        starImports: [{
                name: "NamedExpression",
                expression: { text: "\"test string\"" }
            }]
    });
    it("should have " + i + " imports", function () {
        assert.equal(fileDef.imports.length, i);
    });
});

//# sourceMappingURL=importTests.js.map
