"use strict";
var main_1 = require("./../../../main");
var testHelpers_1 = require("./../../testHelpers");
describe("function name tests", function () {
    var code = "\nfunction myFunction() {\n}\nfunction myFunctionWithParameters(str: string, optionalParam?: string, defaultParam = new Date(), ...restParam: MyClass[]) {\n    return new Date();\n}\nfunction myFunctionWithThisType(this: string, num: number) {\n}\nasync function myAsyncFunction(): any {\n}\nfunction typeGuardFunction(def: any): def is MyClass {\n    return true;\n}\nfunction typeGuardFunctionIntersect(def: any): def is MyClass & MyClass2 {\n    return true;\n}\nfunction *myGeneratorFunction() {\n    yield \"test\";\n}\nclass MyClass {\n}\nclass MyClass2 {\n}\n";
    var def = main_1.getInfoFromString(code);
    testHelpers_1.runFileDefinitionTests(def, {
        functions: [{
                name: "myFunction"
            }, {
                name: "myFunctionWithParameters",
                parameters: [{
                        name: "str",
                        type: { text: "string" }
                    }, {
                        name: "optionalParam",
                        type: { text: "string" },
                        isOptional: true
                    }, {
                        name: "defaultParam",
                        type: { text: "Date" },
                        defaultExpression: { text: "new Date()" },
                        isOptional: true
                    }, {
                        name: "restParam",
                        type: { text: "MyClass[]", isArrayType: true, arrayElementType: { text: "MyClass" } },
                        isOptional: true,
                        isRestParameter: true
                    }],
                returnType: {
                    text: "Date"
                }
            }, {
                name: "myFunctionWithThisType",
                thisType: { text: "string" },
                parameters: [{
                        name: "num",
                        type: { text: "number" }
                    }]
            }, {
                name: "myAsyncFunction",
                isAsync: true,
                returnType: { text: "any" }
            }, {
                name: "typeGuardFunction",
                parameters: [{ name: "def" }],
                returnType: {
                    text: "def is MyClass"
                },
                userDefinedTypeGuard: {
                    parameterName: "def",
                    type: "MyClass"
                }
            }, {
                name: "typeGuardFunctionIntersect",
                parameters: [{ name: "def" }],
                returnType: {
                    text: "def is MyClass & MyClass2"
                },
                userDefinedTypeGuard: {
                    parameterName: "def",
                    type: "MyClass & MyClass2"
                }
            }, {
                name: "myGeneratorFunction",
                isGenerator: true,
                returnType: { text: "IterableIterator<string>" }
            }],
        classes: [{
                name: "MyClass"
            }, {
                name: "MyClass2"
            }]
    });
});

//# sourceMappingURL=functionTests.js.map
