"use strict";
var main_1 = require("./../../../main");
var testHelpers_1 = require("./../../testHelpers");
var definitions_1 = require("./../../../definitions");
describe("module tests", function () {
    var code = "\nmodule MyModule {\n    class MyModuleClass {}\n    export class MyExportedModuleClass {}\n    enum MyModuleEnum {}\n    export enum MyExportedModuleEnum {}\n    function myModuleFunction() {}\n    export function myExportedModuleFunction() {}\n    interface MyModuleInterface {}\n    export interface MyExportedModuleInterface {}\n    module MyInnerModule {\n    }\n    export module MyInnerExportedModule {\n        export class MyInnerModuleClass {}\n        class MyNonExportedInnerClass {}\n    }\n    namespace MyInnerNamespace {\n    }\n    export namespace MyInnerExportedNamespace {\n        export class MyInnerNamespaceClass {}\n    }\n    type myTypeAlias = string;\n    export type myExportedTypeAlias = string;\n}\nexport module MyExportedModule {\n}";
    var def = main_1.getInfoFromString(code);
    testHelpers_1.runFileDefinitionTests(def, {
        namespaces: [{
                name: "MyModule",
                declarationType: definitions_1.NamespaceDeclarationType.Module,
                classes: [
                    { name: "MyModuleClass", order: 0 },
                    { name: "MyExportedModuleClass", isExported: true, order: 1 }
                ],
                enums: [
                    { name: "MyModuleEnum", order: 2 },
                    { name: "MyExportedModuleEnum", isExported: true, order: 3 }
                ],
                functions: [
                    { name: "myModuleFunction", order: 4 },
                    { name: "myExportedModuleFunction", isExported: true, order: 5 }
                ],
                interfaces: [
                    { name: "MyModuleInterface", order: 6 },
                    { name: "MyExportedModuleInterface", isExported: true, order: 7 }
                ],
                namespaces: [{
                        name: "MyInnerModule",
                        declarationType: definitions_1.NamespaceDeclarationType.Module,
                        order: 8
                    }, {
                        name: "MyInnerExportedModule",
                        declarationType: definitions_1.NamespaceDeclarationType.Module,
                        isExported: true,
                        order: 9,
                        classes: [
                            { name: "MyInnerModuleClass", isExported: true, order: 0 },
                            { name: "MyNonExportedInnerClass", order: 1 }
                        ],
                        exports: [{
                                name: "MyInnerModuleClass"
                            }]
                    }, {
                        name: "MyInnerNamespace",
                        declarationType: definitions_1.NamespaceDeclarationType.Namespace,
                        order: 10
                    }, {
                        name: "MyInnerExportedNamespace",
                        declarationType: definitions_1.NamespaceDeclarationType.Namespace,
                        isExported: true,
                        order: 11,
                        classes: [
                            { name: "MyInnerNamespaceClass", isExported: true, order: 0 }
                        ],
                        exports: [{
                                name: "MyInnerNamespaceClass"
                            }]
                    }],
                typeAliases: [{
                        name: "myTypeAlias",
                        type: { text: "string" },
                        order: 12
                    }, {
                        name: "myExportedTypeAlias",
                        type: { text: "string" },
                        isExported: true,
                        order: 13
                    }],
                exports: [{
                        name: "MyInnerExportedModule"
                    }, {
                        name: "MyInnerExportedNamespace"
                    }, {
                        name: "MyExportedModuleClass"
                    }, {
                        name: "MyExportedModuleInterface"
                    }, {
                        name: "MyExportedModuleEnum"
                    }, {
                        name: "myExportedModuleFunction"
                    }, {
                        name: "myExportedTypeAlias"
                    }]
            }, {
                name: "MyExportedModule",
                declarationType: definitions_1.NamespaceDeclarationType.Module,
                isExported: true,
                isNamedExportOfFile: true
            }],
        exports: [{
                name: "MyExportedModule"
            }]
    });
});

//# sourceMappingURL=moduleTests.js.map
