"use strict";
var assert = require("assert");
var path = require("path");
var tsTypeInfo = require("./../main");
var errors_1 = require("./../errors");
describe("Main", function () {
    describe("#getInfoFromFiles()", function () {
        it("should throw an error when not providing an array", function () {
            assert.throws(function () { return tsTypeInfo.getInfoFromFiles(""); }, errors_1.ArgumentTypeError);
        });
        it("should not handle a js file when specifying to not allow them", function () {
            var result = tsTypeInfo.getInfoFromFiles([path.join(__dirname, "../../src/tests/testFiles/nonTsFile.js")], {
                compilerOptions: { allowJs: false }
            });
            assert.equal(result.files.length, 0);
        });
        it("should get the class definition when specifying the compiler option to allow js", function () {
            var result = tsTypeInfo.getInfoFromFiles([path.join(__dirname, "../../src/tests/testFiles/nonTsFile.js")], {
                compilerOptions: { allowJs: true }
            });
            assert.equal(result.files[0].classes[0].name, "MyClass");
        });
        it("should throw an error when a file isn't found", function () {
            assert.throws(function () {
                tsTypeInfo.getInfoFromFiles(["nonExistentFile.txt"]);
            }, errors_1.FileNotFoundError, "File not found: nonExistentFile.txt");
        });
    });
    describe("#getInfoFromString()", function () {
        it("should throw an error when not providing a string", function () {
            assert.throws(function () { return tsTypeInfo.getInfoFromString([]); }, errors_1.ArgumentTypeError);
        });
        it("should allow changing the compiler options", function () {
            var result = tsTypeInfo.getInfoFromString("class MyClass {}\n\n", {});
            assert.equal(result.classes[0].name, "MyClass");
        });
    });
});

//# sourceMappingURL=mainTests.js.map
