"use strict";
var assert = require("assert");
var indexOfAllDefinitionsInText_1 = require("./../../../utils/renameDefinitionAs/indexOfAllDefinitionsInText");
describe("indexOfAllDefinitionsInText", function () {
    it("should find the name outside type parameters", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyClass<any, MyOtherClass>", "MyClass");
        assert.deepEqual(result, [0]);
    });
    it("should find the name inside type parameters", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyClass<any, MyOtherClass>", "MyOtherClass");
        assert.deepEqual(result, [13]);
    });
    it("should find multiple names", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyClass<any, MyClass>", "MyClass");
        assert.deepEqual(result, [0, 13]);
    });
    it("should find at beginning of string", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyClass1 | MyClass2", "MyClass1");
        assert.deepEqual(result, [0]);
    });
    it("should find at end of string", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyClass1 | MyClass2", "MyClass2");
        assert.deepEqual(result, [11]);
    });
    it("should find in intersection and union type", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("(MyClass1 & MyClass2) | MyClass2", "MyClass2");
        assert.deepEqual(result, [12, 24]);
    });
    it("should not find a definition when it doesn't match exactly on the left", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyTestClass", "TestClass");
        assert.deepEqual(result, []);
    });
    it("should not rename a definition when it doesn't match exactly on the right", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyTestClass", "MyTest");
        assert.deepEqual(result, []);
    });
    it("should find a namespace when it matches exactly", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyNamespace.MyClass", "MyNamespace.MyClass");
        assert.deepEqual(result, [0]);
    });
    it("should find a namespace when it matches from the left", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyNamespace.MyOtherNamespace.MyClass", "MyNamespace.MyOtherNamespace");
        assert.deepEqual(result, [0]);
    });
    it("should NOT find a namespace when it matches from the right", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("MyNamespace.MyOtherNamespace.MyClass", "MyOtherNamespace.MyClass");
        assert.deepEqual(result, []);
    });
    it("should NOT find in single quotes", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("'MyNamespace'", "MyNamespace");
        assert.deepEqual(result, []);
    });
    it("should NOT find in double quotes", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("\"MyNamespace\"", "MyNamespace");
        assert.deepEqual(result, []);
    });
    it("should NOT find in back tick quotes", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("`MyNamespace`", "MyNamespace");
        assert.deepEqual(result, []);
    });
    it("should find in back tick quotes' ${}", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("`MyNamespace${MyNamespace}MyNamespace`", "MyNamespace");
        assert.deepEqual(result, [14]);
    });
    it("should handle backslashes in single quotes", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("'MyNamespace\\'MyNamespace'", "MyNamespace");
        assert.deepEqual(result, []);
    });
    it("should handle backslashes in double quotes", function () {
        var result = indexOfAllDefinitionsInText_1.indexOfAllDefinitionsInText("\"MyNamespace\\\"MyNamespace\"", "MyNamespace");
        assert.deepEqual(result, []);
    });
});

//# sourceMappingURL=indexOfAllDefinitionsInTextTests.js.map
