"use strict";
var assert = require("assert");
var renameDefinitionInText_1 = require("./../../../utils/renameDefinitionAs/renameDefinitionInText");
describe("renameDefinitionInText()", function () {
    it("should rename the name outside type parameters", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyClass<any, MyOtherClass>", "MyClass", "MyNewName");
        assert.equal(result, "MyNewName<any, MyOtherClass>");
    });
    it("should rename the name inside type parameters", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyClass<any, MyOtherClass>", "MyOtherClass", "MyNewName");
        assert.equal(result, "MyClass<any, MyNewName>");
    });
    it("should rename at beginning of string", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyClass1 | MyClass2", "MyClass1", "MyNewName");
        assert.equal(result, "MyNewName | MyClass2");
    });
    it("should rename at end of string", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyClass1 | MyClass2", "MyClass2", "MyNewName");
        assert.equal(result, "MyClass1 | MyNewName");
    });
    it("should rename in intersection and union type", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("(MyClass1 & MyClass2) | MyClass2", "MyClass2", "MyNewName");
        assert.equal(result, "(MyClass1 & MyNewName) | MyNewName");
    });
    it("should not rename a definition when it doesn't match exactly on the left", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyTestClass", "TestClass", "NewTestClass");
        assert.equal(result, "MyTestClass");
    });
    it("should not rename a definition when it doesn't match exactly on the right", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyTestClass", "MyTest", "NewMyTest");
        assert.equal(result, "MyTestClass");
    });
    it("should rename a namespace when it matches exactly", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyNamespace.MyClass", "MyNamespace.MyClass", "MyNamespace.MyNewName");
        assert.equal(result, "MyNamespace.MyNewName");
    });
    it("should rename a namespace when it matches from the left", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyNamespace.MyOtherNamespace.MyClass", "MyNamespace.MyOtherNamespace", "MyNamespace.MyNewNamespace");
        assert.equal(result, "MyNamespace.MyNewNamespace.MyClass");
    });
    it("should NOT rename a namespace when it matches from the right", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("MyNamespace.MyOtherNamespace.MyClass", "MyOtherNamespace.MyClass", "MyOtherNamespace.MyNewClass");
        assert.equal(result, "MyNamespace.MyOtherNamespace.MyClass");
    });
    it("should NOT rename in single quotes", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("'MyNamespace'", "MyNamespace", "MyNewNamespace");
        assert.equal(result, "'MyNamespace'");
    });
    it("should NOT rename in double quotes", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("\"MyNamespace\"", "MyNamespace", "MyNewNamespace");
        assert.equal(result, "\"MyNamespace\"");
    });
    it("should NOT rename in back tick quotes", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("`MyNamespace`", "MyNamespace", "MyNewNamespace");
        assert.equal(result, "`MyNamespace`");
    });
    it("should rename in back tick quotes' ${}", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("`MyNamespace${MyNamespace}MyNamespace`", "MyNamespace", "MyNewNamespace");
        assert.equal(result, "`MyNamespace${MyNewNamespace}MyNamespace`");
    });
    it("should handle backslashes in single quotes", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("\"MyNamespace\\\"MyNamespace\"", "MyNamespace", "MyNewNamespace");
        assert.equal(result, "\"MyNamespace\\\"MyNamespace\"");
    });
    it("should handle backslashes in double quotes", function () {
        var result = renameDefinitionInText_1.renameDefinitionInText("'MyNamespace\\'MyNamespace'", "MyNamespace", "MyNewNamespace");
        assert.equal(result, "'MyNamespace\\'MyNamespace'");
    });
});

//# sourceMappingURL=renameDefinitionInTextTests.js.map
