"use strict";
var definitions = require("./definitions");
var writers_1 = require("./writers");
var WriteFlags_1 = require("./WriteFlags");
var utils_1 = require("./utils");
function writeDefinition(definition, writer, writeFlags) {
    if (writeFlags === void 0) { writeFlags = WriteFlags_1.WriteFlags.Default; }
    if (definition instanceof definitions.ClassDefinition) {
        new writers_1.ClassWriter(writer).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.InterfaceDefinition) {
        new writers_1.InterfaceWriter(writer).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.FunctionDefinition) {
        new writers_1.FunctionWriter(writer).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.FileDefinition) {
        new writers_1.FileWriter(writer).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.NamespaceDefinition) {
        new writers_1.NamespaceWriter(writer, new writers_1.ModuledWriter(writer)).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.EnumDefinition) {
        new writers_1.EnumWriter(writer).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.TypeAliasDefinition) {
        new writers_1.TypeAliasWriter(writer).write(definition, writeFlags);
    }
    else if (definition instanceof definitions.VariableDefinition) {
        new writers_1.VariableWriter(writer).write(definition, writeFlags);
    }
    else {
        utils_1.Logger.warn("Not implemented writer for definition: " + definition.name);
    }
}
exports.writeDefinition = writeDefinition;

//# sourceMappingURL=writeDefinition.js.map
