"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var definitions_1 = require("./../definitions");
var WriteFlags_1 = require("./../WriteFlags");
var BaseWriter_1 = require("./BaseWriter");
var BaseDefinitionWriter = (function (_super) {
    __extends(BaseDefinitionWriter, _super);
    function BaseDefinitionWriter() {
        _super.apply(this, arguments);
    }
    BaseDefinitionWriter.prototype.write = function (def, flags) {
        if (typeof def.onBeforeWrite === "function") {
            def.onBeforeWrite(this.writer);
            this.writer.newLineIfLastNotNewLine();
        }
        this.writeDefault(def, flags);
        if (typeof def.onAfterWrite === "function") {
            def.onAfterWrite(this.writer);
            this.writer.newLineIfLastNotNewLine();
        }
    };
    BaseDefinitionWriter.prototype.writeAsyncKeyword = function (def) {
        this.writer.conditionalWrite(def.isAsync, "async ");
    };
    BaseDefinitionWriter.prototype.writeConstKeyword = function (def) {
        this.writer.conditionalWrite(def.isConst, "const ");
    };
    BaseDefinitionWriter.prototype.writeDeclareKeyword = function (def) {
        this.writer.conditionalWrite(def.hasDeclareKeyword, "declare ");
    };
    BaseDefinitionWriter.prototype.writeExportKeyword = function (def, flags) {
        if ((def.isExported && !def.isDefaultExportOfFile) || def.isNamedExportOfFile) {
            var shouldWrite = false;
            if (def instanceof definitions_1.InterfaceDefinition || def instanceof definitions_1.TypeAliasDefinition) {
                shouldWrite = (flags & WriteFlags_1.WriteFlags.IsInAmbientContext) === 0;
            }
            else {
                shouldWrite = !def.isAmbient;
            }
            if (shouldWrite) {
                this.writer.write("export ");
            }
        }
    };
    return BaseDefinitionWriter;
}(BaseWriter_1.BaseWriter));
exports.BaseDefinitionWriter = BaseDefinitionWriter;

//# sourceMappingURL=BaseDefinitionWriter.js.map
