"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var definitions = require("./../definitions");
var WriteFlags_1 = require("./../WriteFlags");
var BaseDefinitionWriter_1 = require("./BaseDefinitionWriter");
var ExtendsImplementsClauseWriter_1 = require("./ExtendsImplementsClauseWriter");
var ClassConstructorWriter_1 = require("./ClassConstructorWriter");
var TypeParametersWriter_1 = require("./TypeParametersWriter");
var PropertyWriter_1 = require("./PropertyWriter");
var MethodWriter_1 = require("./MethodWriter");
var FunctionBodyWriter_1 = require("./FunctionBodyWriter");
var ClassWriter = (function (_super) {
    __extends(ClassWriter, _super);
    function ClassWriter() {
        _super.apply(this, arguments);
        this.typeParametersWriter = new TypeParametersWriter_1.TypeParametersWriter(this.writer);
        this.propertyWriter = new PropertyWriter_1.PropertyWriter(this.writer);
        this.methodWriter = new MethodWriter_1.MethodWriter(this.writer);
        this.classConstructorWriter = new ClassConstructorWriter_1.ClassConstructorWriter(this.writer);
    }
    ClassWriter.prototype.writeDefault = function (def, flags) {
        var _this = this;
        this.writeHeader(def, flags);
        this.writer.block(function () {
            _this.writeStaticProperties(def, flags);
            _this.writer.newLine();
            _this.writeStaticMethods(def, flags);
            _this.writer.newLine();
            _this.writeProperties(def, flags);
            _this.writer.newLine();
            _this.writeConstructor(def.constructorDef, flags);
            _this.writer.newLine();
            _this.writeMethods(def, flags);
        });
    };
    ClassWriter.prototype.writeHeader = function (def, flags) {
        this.writeExportKeyword(def, flags);
        this.writeDeclareKeyword(def);
        this.writeAbstract(def);
        this.writer.write("class ").write(def.name);
        this.typeParametersWriter.write(def.typeParameters, flags);
        var extendsImplementsWriter = new ExtendsImplementsClauseWriter_1.ExtendsImplementsClauseWriter(this.writer);
        extendsImplementsWriter.writeExtends(def).writeImplements(def);
    };
    ClassWriter.prototype.writeAbstract = function (def) {
        if (def.isAbstract) {
            this.writer.write("abstract ");
        }
    };
    ClassWriter.prototype.writeConstructor = function (constructorDef, flags) {
        var _this = this;
        if (constructorDef != null && ClassConstructorWriter_1.ClassConstructorWriter.shouldWriteConstructor(constructorDef, flags)) {
            var willWriteFunctionBody = FunctionBodyWriter_1.FunctionBodyWriter.willWriteFunctionBody(constructorDef, flags);
            if (!willWriteFunctionBody) {
                flags |= WriteFlags_1.WriteFlags.HideScopeOnParameters;
            }
            this.classConstructorWriter.write(constructorDef, flags);
            this.writer.newLine();
            // if the function body won't be written, the scoped constructor parameters need to be written out as properties
            if (!willWriteFunctionBody) {
                (constructorDef.parameters || []).filter(function (p) { return p.scope !== definitions.ClassConstructorParameterScope.None; }).forEach(function (p) {
                    _this.propertyWriter.write(p.toClassProperty(), flags);
                });
            }
        }
    };
    ClassWriter.prototype.writeStaticProperties = function (def, flags) {
        var _this = this;
        def.staticProperties.forEach(function (p) {
            if (_this.shouldInclude(p, flags)) {
                _this.propertyWriter.write(p, flags);
            }
        });
    };
    ClassWriter.prototype.writeProperties = function (def, flags) {
        var _this = this;
        def.properties.forEach(function (p) {
            if (_this.shouldInclude(p, flags) && !p.isConstructorParameter) {
                var willWriteAccessorBody = PropertyWriter_1.PropertyWriter.willWriteAccessorBody(p);
                if (willWriteAccessorBody) {
                    _this.writer.newLine();
                }
                _this.propertyWriter.write(p, flags);
                if (willWriteAccessorBody) {
                    _this.writer.newLine();
                }
            }
        });
    };
    ClassWriter.prototype.writeStaticMethods = function (def, flags) {
        var _this = this;
        if (def.isAmbient) {
            flags = flags | WriteFlags_1.WriteFlags.HideFunctionBodies;
        }
        def.staticMethods.forEach(function (m) { return _this.writeMethod(m, flags); });
    };
    ClassWriter.prototype.writeMethods = function (def, flags) {
        var _this = this;
        if (def.isAmbient) {
            flags = flags | WriteFlags_1.WriteFlags.HideFunctionBodies;
        }
        def.methods.forEach(function (m) { return _this.writeMethod(m, flags); });
    };
    ClassWriter.prototype.writeMethod = function (def, flags) {
        var thisHasBlankLine = FunctionBodyWriter_1.FunctionBodyWriter.willWriteFunctionBody(def, flags);
        if (thisHasBlankLine) {
            this.writer.newLine();
        }
        if (this.shouldInclude(def, flags)) {
            this.methodWriter.write(def, flags);
        }
        if (thisHasBlankLine) {
            this.writer.newLine();
        }
    };
    ClassWriter.prototype.shouldInclude = function (def, flags) {
        if (def.scope === definitions.Scope.Private && (flags & WriteFlags_1.WriteFlags.HidePrivateMembers)) {
            return false;
        }
        else if (def.scope === definitions.Scope.Protected && (flags & WriteFlags_1.WriteFlags.HideProtectedMembers)) {
            return false;
        }
        else {
            return true;
        }
    };
    return ClassWriter;
}(BaseDefinitionWriter_1.BaseDefinitionWriter));
exports.ClassWriter = ClassWriter;

//# sourceMappingURL=ClassWriter.js.map
