"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var WriteFlags_1 = require("./../WriteFlags");
var CallSignatureWriter_1 = require("./CallSignatureWriter");
var TypeParametersWriter_1 = require("./TypeParametersWriter");
var ParametersWriter_1 = require("./ParametersWriter");
var BaseDefinitionWriter_1 = require("./BaseDefinitionWriter");
var FunctionBodyWriter_1 = require("./FunctionBodyWriter");
var FunctionReturnTypeWriter_1 = require("./FunctionReturnTypeWriter");
var FunctionWriter = (function (_super) {
    __extends(FunctionWriter, _super);
    function FunctionWriter() {
        _super.apply(this, arguments);
        this.callSignatureWriter = new CallSignatureWriter_1.CallSignatureWriter(this.writer);
        this.typeParametersWriter = new TypeParametersWriter_1.TypeParametersWriter(this.writer);
        this.parametersWriter = new ParametersWriter_1.ParametersWriter(this.writer);
        this.functionBodyWriter = new FunctionBodyWriter_1.FunctionBodyWriter(this.writer);
        this.functionReturnTypeWriter = new FunctionReturnTypeWriter_1.FunctionReturnTypeWriter(this.writer);
    }
    FunctionWriter.prototype.writeDefault = function (def, flags) {
        var _this = this;
        (def.overloadSignatures || []).forEach(function (s) {
            _this.writeStartOfFunctionHeader(def, flags);
            _this.callSignatureWriter.write(s, flags);
        });
        this.writeImplementation(def, flags);
        this.writer.newLineIfLastNotNewLine();
    };
    FunctionWriter.prototype.writeImplementation = function (def, flags) {
        var showImplementation = def.overloadSignatures.length === 0 || (flags & WriteFlags_1.WriteFlags.HideFunctionImplementations) === 0;
        if (showImplementation) {
            this.writeStartOfFunctionHeader(def, flags);
            this.typeParametersWriter.write(def.typeParameters, flags);
            this.parametersWriter.write(def, flags);
            this.functionReturnTypeWriter.write(def, flags);
            this.functionBodyWriter.write(def, flags);
        }
    };
    FunctionWriter.prototype.writeStartOfFunctionHeader = function (def, flags) {
        this.writeExportKeyword(def, flags);
        this.writeDeclareKeyword(def);
        this.writeAsyncKeyword(def);
        this.writer.write("function ");
        this.writer.conditionalWrite(def.isGenerator, "*");
        this.writer.write(def.name);
    };
    return FunctionWriter;
}(BaseDefinitionWriter_1.BaseDefinitionWriter));
exports.FunctionWriter = FunctionWriter;

//# sourceMappingURL=FunctionWriter.js.map
