"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var BaseDefinitionWriter_1 = require("./BaseDefinitionWriter");
var ImportWriter = (function (_super) {
    __extends(ImportWriter, _super);
    function ImportWriter() {
        _super.apply(this, arguments);
    }
    ImportWriter.prototype.writeDefault = function (def) {
        var hasDefaultImport = def.defaultImport != null;
        var hasStarImport = def.starImportName != null && def.starImportName.length > 0;
        var hasNamedImports = (def.namedImports || []).length > 0;
        this.writer.write("import ");
        if (hasDefaultImport) {
            this.writeDefaultImport(def);
            this.writer.conditionalWrite(hasStarImport || hasNamedImports, ", ");
        }
        if (hasStarImport) {
            this.writeStarImport(def);
        }
        else if (hasNamedImports) {
            this.writeNamedImports(def);
        }
        if (hasDefaultImport || hasStarImport || hasNamedImports) {
            this.writer.write(" from ");
        }
        this.writer.write("\"" + def.moduleSpecifier + "\";");
        this.writer.newLine();
    };
    ImportWriter.prototype.writeStarImport = function (def) {
        this.writer.write("* as " + def.starImportName);
    };
    ImportWriter.prototype.writeDefaultImport = function (def) {
        if (def.defaultImport != null) {
            this.writer.write(def.defaultImport.name);
        }
    };
    ImportWriter.prototype.writeNamedImports = function (def) {
        var _this = this;
        this.writer.write("{");
        (def.namedImports || []).forEach(function (namedImport, i) {
            var name = namedImport.name, alias = namedImport.alias;
            _this.writer.conditionalWrite(i !== 0, ", ");
            if (alias != null && name !== alias) {
                _this.writer.write(name + " as " + alias);
            }
            else {
                _this.writer.write("" + name);
            }
        });
        this.writer.write("}");
    };
    return ImportWriter;
}(BaseDefinitionWriter_1.BaseDefinitionWriter));
exports.ImportWriter = ImportWriter;

//# sourceMappingURL=ImportWriter.js.map
