"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var BaseDefinitionWriter_1 = require("./BaseDefinitionWriter");
var CallSignatureWriter_1 = require("./CallSignatureWriter");
var IndexSignatureWriter_1 = require("./IndexSignatureWriter");
var ExtendsImplementsClauseWriter_1 = require("./ExtendsImplementsClauseWriter");
var TypeParametersWriter_1 = require("./TypeParametersWriter");
var PropertyWriter_1 = require("./PropertyWriter");
var MethodWriter_1 = require("./MethodWriter");
var InterfaceWriter = (function (_super) {
    __extends(InterfaceWriter, _super);
    function InterfaceWriter() {
        _super.apply(this, arguments);
        this.typeParametersWriter = new TypeParametersWriter_1.TypeParametersWriter(this.writer);
        this.propertyWriter = new PropertyWriter_1.PropertyWriter(this.writer);
        this.methodWriter = new MethodWriter_1.MethodWriter(this.writer);
        this.callSignatureWriter = new CallSignatureWriter_1.CallSignatureWriter(this.writer);
        this.indexSignatureWriter = new IndexSignatureWriter_1.IndexSignatureWriter(this.writer);
    }
    InterfaceWriter.prototype.writeDefault = function (def, flags) {
        var _this = this;
        this.writeHeader(def, flags);
        this.writer.block(function () {
            _this.writeNewSignatures(def, flags);
            _this.writeCallSignatures(def, flags);
            _this.writeIndexSignatures(def, flags);
            _this.writeProperties(def, flags);
            _this.writer.newLine();
            _this.writeMethods(def, flags);
        });
    };
    InterfaceWriter.prototype.writeHeader = function (def, flags) {
        this.writeExportKeyword(def, flags);
        this.writeDeclareKeyword(def);
        this.writer.write("interface ").write(def.name);
        this.typeParametersWriter.write(def.typeParameters, flags);
        var extendsImplementsWriter = new ExtendsImplementsClauseWriter_1.ExtendsImplementsClauseWriter(this.writer);
        extendsImplementsWriter.writeExtends(def);
    };
    InterfaceWriter.prototype.writeNewSignatures = function (def, flags) {
        var _this = this;
        def.newSignatures.forEach(function (n) {
            _this.writer.write("new");
            _this.callSignatureWriter.write(n, flags);
        });
    };
    InterfaceWriter.prototype.writeCallSignatures = function (def, flags) {
        var _this = this;
        def.callSignatures.forEach(function (c) {
            _this.callSignatureWriter.write(c, flags);
        });
    };
    InterfaceWriter.prototype.writeIndexSignatures = function (def, flags) {
        var _this = this;
        def.indexSignatures.forEach(function (s) {
            _this.indexSignatureWriter.write(s, flags);
        });
    };
    InterfaceWriter.prototype.writeProperties = function (def, flags) {
        var _this = this;
        def.properties.forEach(function (p) {
            _this.propertyWriter.write(p, flags);
        });
    };
    InterfaceWriter.prototype.writeMethods = function (def, flags) {
        var _this = this;
        def.methods.forEach(function (m) {
            _this.methodWriter.write(m, flags);
        });
    };
    return InterfaceWriter;
}(BaseDefinitionWriter_1.BaseDefinitionWriter));
exports.InterfaceWriter = InterfaceWriter;

//# sourceMappingURL=InterfaceWriter.js.map
