"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var definitions = require("./../definitions");
var utils_1 = require("./../utils");
var WriteFlags_1 = require("./../WriteFlags");
var BaseWriter_1 = require("./BaseWriter");
var NamespaceWriter_1 = require("./NamespaceWriter");
var ClassWriter_1 = require("./ClassWriter");
var EnumWriter_1 = require("./EnumWriter");
var InterfaceWriter_1 = require("./InterfaceWriter");
var FunctionWriter_1 = require("./FunctionWriter");
var VariableWriter_1 = require("./VariableWriter");
var TypeAliasWriter_1 = require("./TypeAliasWriter");
var ModuledWriter = (function (_super) {
    __extends(ModuledWriter, _super);
    function ModuledWriter() {
        _super.apply(this, arguments);
        this.namespaceWriter = new NamespaceWriter_1.NamespaceWriter(this.writer, this);
        this.interfaceWriter = new InterfaceWriter_1.InterfaceWriter(this.writer);
        this.classWriter = new ClassWriter_1.ClassWriter(this.writer);
        this.enumWriter = new EnumWriter_1.EnumWriter(this.writer);
        this.functionWriter = new FunctionWriter_1.FunctionWriter(this.writer);
        this.variableWriter = new VariableWriter_1.VariableWriter(this.writer);
        this.typeAliasWriter = new TypeAliasWriter_1.TypeAliasWriter(this.writer);
    }
    ModuledWriter.prototype.write = function (def, flags) {
        if (def instanceof definitions.FileDefinition && utils_1.DefinitionUtils.isDefinitionFile(def) || def instanceof definitions.NamespaceDefinition && def.isAmbient) {
            flags = flags | WriteFlags_1.WriteFlags.IsInAmbientContext;
        }
        else {
            flags = flags & ~WriteFlags_1.WriteFlags.IsInAmbientContext;
        }
        this.writeChildren(def, flags);
    };
    ModuledWriter.prototype.writeChildren = function (def, flags) {
        var _this = this;
        // order these by what should be written if order is null
        var allDefinitions = def.typeAliases.concat(def.interfaces, def.enums, def.classes, def.namespaces, def.variables, def.functions);
        allDefinitions.sort(function (a, b) {
            if (b.order == null) {
                return -1;
            }
            else if (a.order == null) {
                return 1;
            }
            else {
                return a.order - b.order;
            }
        });
        allDefinitions.forEach(function (d, i) {
            if (i > 0 && !(d instanceof definitions.TypeAliasDefinition) && !(d instanceof definitions.VariableDefinition)) {
                _this.writer.newLine();
            }
            if (d instanceof definitions.ClassDefinition) {
                _this.classWriter.write(d, flags);
            }
            else if (d instanceof definitions.InterfaceDefinition) {
                _this.interfaceWriter.write(d, flags);
            }
            else if (d instanceof definitions.FunctionDefinition) {
                _this.functionWriter.write(d, flags);
            }
            else if (d instanceof definitions.NamespaceDefinition) {
                _this.namespaceWriter.write(d, flags);
            }
            else if (d instanceof definitions.VariableDefinition) {
                if (i > 0 && !(allDefinitions[i - 1] instanceof definitions.VariableDefinition)) {
                    _this.writer.newLine();
                }
                _this.variableWriter.write(d, flags);
            }
            else if (d instanceof definitions.EnumDefinition) {
                _this.enumWriter.write(d, flags);
            }
            else if (d instanceof definitions.TypeAliasDefinition) {
                if (i > 0 && !(allDefinitions[i - 1] instanceof definitions.TypeAliasDefinition)) {
                    _this.writer.newLine();
                }
                _this.typeAliasWriter.write(d, flags);
            }
        });
    };
    return ModuledWriter;
}(BaseWriter_1.BaseWriter));
exports.ModuledWriter = ModuledWriter;

//# sourceMappingURL=ModuledWriter.js.map
