"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var WriteFlags_1 = require("./../WriteFlags");
var definitions_1 = require("./../definitions");
var BaseWriter_1 = require("./BaseWriter");
var ParameterWriter_1 = require("./ParameterWriter");
var ParameterWithDestructuringWriter_1 = require("./ParameterWithDestructuringWriter");
var TypeWriter_1 = require("./TypeWriter");
var ParametersWriter = (function (_super) {
    __extends(ParametersWriter, _super);
    function ParametersWriter() {
        _super.apply(this, arguments);
        this.parameterWriter = new ParameterWriter_1.ParameterWriter(this.writer);
        this.parameterWithDestructuringWriter = new ParameterWithDestructuringWriter_1.ParameterWithDestructuringWriter(this.writer);
        this.typeWriter = new TypeWriter_1.TypeWriter(this.writer);
    }
    ParametersWriter.prototype.write = function (def, flags) {
        var _this = this;
        var thisType = def.thisType;
        this.writer.write("(");
        this.writeThisType(thisType);
        def.parameters.forEach(function (param, i) {
            _this.writer.conditionalWrite(i > 0 || thisType != null, ", ");
            if (param.destructuringProperties.length === 0) {
                if (param instanceof definitions_1.ClassConstructorParameterDefinition) {
                    if ((flags & WriteFlags_1.WriteFlags.HideScopeOnParameters) === 0) {
                        _this.writeScope(param.scope, flags);
                        _this.writer.conditionalWrite(param.isReadonly, "readonly ");
                    }
                }
                _this.parameterWriter.write(param, flags);
            }
            else {
                _this.parameterWithDestructuringWriter.write(param, flags);
            }
        });
        this.writer.write(")");
    };
    ParametersWriter.prototype.writeThisType = function (thisType) {
        if (thisType != null) {
            this.writer.write("this");
            this.typeWriter.writeWithColon(thisType);
        }
    };
    ParametersWriter.prototype.writeScope = function (scope, flags) {
        if (scope === definitions_1.ClassConstructorParameterScope.Private) {
            this.writer.write("private ");
        }
        else if (scope === definitions_1.ClassConstructorParameterScope.Protected) {
            this.writer.write("protected ");
        }
        else if (scope === definitions_1.ClassConstructorParameterScope.Public) {
            this.writer.write("public ");
        }
    };
    return ParametersWriter;
}(BaseWriter_1.BaseWriter));
exports.ParametersWriter = ParametersWriter;

//# sourceMappingURL=ParametersWriter.js.map
