"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var definitions_1 = require("./../definitions");
var TypeWriter_1 = require("./TypeWriter");
var ScopeWriter_1 = require("./ScopeWriter");
var BaseDefinitionWriter_1 = require("./BaseDefinitionWriter");
var TypeWithDefaultExpressionWriter_1 = require("./TypeWithDefaultExpressionWriter");
var PropertyWriter = (function (_super) {
    __extends(PropertyWriter, _super);
    function PropertyWriter() {
        _super.apply(this, arguments);
        this.typeWriter = new TypeWriter_1.TypeWriter(this.writer);
        this.scopeWriter = new ScopeWriter_1.ScopeWriter(this.writer);
        this.typeWithDefaultExpressionWriter = new TypeWithDefaultExpressionWriter_1.TypeWithDefaultExpressionWriter(this.writer);
    }
    PropertyWriter.willWriteAccessorBody = function (def) {
        return PropertyWriter.isAccessor(def) && (def.onWriteGetBody != null || def.onWriteSetBody != null);
    };
    PropertyWriter.isAbstractAccessor = function (def) {
        return PropertyWriter.isAccessor(def) && def.isAbstract;
    };
    PropertyWriter.isAccessor = function (def) {
        return def instanceof definitions_1.ClassPropertyDefinition && def.kind !== definitions_1.ClassPropertyKind.Normal;
    };
    PropertyWriter.prototype.writeDefault = function (def, flags) {
        if (PropertyWriter.willWriteAccessorBody(def) || PropertyWriter.isAbstractAccessor(def)) {
            this.writeAccessor(def);
        }
        else {
            this.writeNormalProperty(def, flags);
        }
    };
    PropertyWriter.prototype.writeAccessor = function (def) {
        if (def.kind & definitions_1.ClassPropertyKind.GetAccessor) {
            this.writeGetAccessor(def);
        }
        if (def.kind & definitions_1.ClassPropertyKind.SetAccessor) {
            this.writer.conditionalNewLine(!def.isAbstract && def.kind === definitions_1.ClassPropertyKind.GetSetAccessor);
            this.writeSetAccessor(def);
        }
    };
    PropertyWriter.prototype.writeGetAccessor = function (def) {
        var _this = this;
        this.writeCommonHeader(def);
        this.writer.write("get ");
        this.writer.write(def.name);
        this.writer.write("()");
        this.typeWriter.writeWithColon(def.type);
        if (def.isAbstract) {
            this.writer.write(";").newLine();
        }
        else {
            this.writer.block(function () {
                if (typeof def.onWriteGetBody === "function") {
                    def.onWriteGetBody(_this.writer);
                }
            });
        }
    };
    PropertyWriter.prototype.writeSetAccessor = function (def) {
        var _this = this;
        this.writeCommonHeader(def);
        this.writer.write("set ");
        this.writer.write(def.name);
        this.writer.write("(value"); // default to value for now
        this.typeWriter.writeWithColon(def.type);
        this.writer.write(")");
        if (def.isAbstract) {
            this.writer.write(";").newLine();
        }
        else {
            this.writer.block(function () {
                if (typeof def.onWriteSetBody === "function") {
                    def.onWriteSetBody(_this.writer);
                }
            });
        }
    };
    PropertyWriter.prototype.writeNormalProperty = function (def, flags) {
        this.writeCommonHeader(def);
        this.writer.write(def.name);
        this.writer.conditionalWrite(def.isOptional, "?");
        if (def instanceof definitions_1.InterfacePropertyDefinition) {
            this.typeWriter.writeWithColon(def.type);
        }
        else {
            this.typeWithDefaultExpressionWriter.writeWithOptionalCheck(def, flags);
        }
        this.writer.write(";").newLine();
    };
    PropertyWriter.prototype.writeCommonHeader = function (def) {
        this.scopeWriter.write(def.scope);
        this.writer.spaceIfLastNotSpace();
        this.writer.conditionalWrite(def.isAbstract, "abstract ");
        this.writer.conditionalWrite(def instanceof definitions_1.ClassStaticPropertyDefinition, "static ");
        this.writer.conditionalWrite(def.isReadonly, "readonly ");
    };
    return PropertyWriter;
}(BaseDefinitionWriter_1.BaseDefinitionWriter));
exports.PropertyWriter = PropertyWriter;

//# sourceMappingURL=PropertyWriter.js.map
